from typing import Dict

import pytest

from litestar import Controller, HttpMethod, Litestar, Router, get, post
from litestar.datastructures import CacheControlHeader, ETag, ResponseHeader
from litestar.datastructures.headers import Header
from litestar.status_codes import HTTP_201_CREATED
from litestar.testing import TestClient, create_test_client


def test_response_headers() -> None:
    router_first = ResponseHeader(name="second", value="1")
    router_second = ResponseHeader(name="third", value="2")
    controller_first = ResponseHeader(name="first", value="3")
    controller_second = ResponseHeader(name="second", value="4")
    app_first = ResponseHeader(name="first", value="5")
    app_second = ResponseHeader(name="fourth", value="6")
    local_first = ResponseHeader(name="first", value="7")

    test_path = "/test"

    class MyController(Controller):
        path = test_path
        response_headers = [controller_first, controller_second]

        @get(path="/{path_param:str}", response_headers=[local_first])
        def test_method(self) -> None:
            pass

    first_router = Router(path="/users", response_headers=[router_first, router_second], route_handlers=[MyController])
    second_router = Router(
        path="/external", response_headers=[ResponseHeader(name="external", value="nope")], route_handlers=[]
    )
    app = Litestar(
        openapi_config=None,
        response_headers=[app_first, app_second],
        route_handlers=[first_router, second_router],
    )

    route_handler, _ = app.routes[0].route_handler_map[HttpMethod.GET]  # type: ignore[union-attr]
    resolved_headers = {header.name: header for header in route_handler.resolve_response_headers()}
    assert resolved_headers["first"].value == local_first.value
    assert resolved_headers["second"].value == controller_second.value
    assert resolved_headers["third"].value == router_second.value
    assert resolved_headers["fourth"].value == app_second.value
    assert "external" not in resolved_headers


def test_response_headers_mapping() -> None:
    @get(response_headers={"foo": "bar"})
    def handler_one() -> None:
        pass

    @get(response_headers=[ResponseHeader(name="foo", value="bar")])
    def handler_two() -> None:
        pass

    assert handler_one.resolve_response_headers() == handler_two.resolve_response_headers()


def test_response_headers_mapping_unresolved() -> None:
    # this should never happen, as there's no way to create this situation which type-checks.
    # we test for it nevertheless

    @get()
    def handler_one() -> None:
        pass

    handler_one.response_headers = {"foo": "bar"}  # type: ignore[assignment]

    assert handler_one.resolve_response_headers() == frozenset([ResponseHeader(name="foo", value="bar")])


def test_response_headers_rendering() -> None:
    @post(
        path="/test",
        tags=["search"],
        response_headers=[ResponseHeader(name="test-header", value="test value", description="test")],
    )
    def my_handler(data: Dict[str, str]) -> Dict[str, str]:
        return data

    with create_test_client(my_handler) as client:
        response = client.post("/test", json={"hello": "world"})
        assert response.status_code == HTTP_201_CREATED
        assert response.headers.get("test-header") == "test value"


@pytest.mark.parametrize(
    "config_kwarg,app_header,controller_header,handler_header",
    [
        (
            "etag",
            ETag(value="1"),
            ETag(value="2"),
            ETag(value="3"),
        ),
        (
            "cache_control",
            CacheControlHeader(max_age=1),
            CacheControlHeader(max_age=2),
            CacheControlHeader(max_age=3),
        ),
    ],
)
def test_explicit_response_headers(
    config_kwarg: str, app_header: Header, controller_header: Header, handler_header: Header
) -> None:
    class MyController(Controller):
        @get(
            path="/handler-override",
            **{config_kwarg: handler_header},  # type: ignore[arg-type]
        )
        def controller_override(self) -> None:
            pass

        @get(path="/controller")
        def controller_handler(self) -> None:
            pass

    setattr(MyController, config_kwarg, controller_header)

    @get(path="/app")
    def app_handler() -> None:
        pass

    app = Litestar(
        route_handlers=[MyController, app_handler],
        **{config_kwarg: app_header},  # type: ignore[arg-type]
    )

    with TestClient(app=app) as client:
        for path, expected_value in {
            "handler-override": handler_header,
            "controller": controller_header,
            "app": app_header,
        }.items():
            response = client.get(path)
            assert response.headers[expected_value.HEADER_NAME] == expected_value.to_header()


@pytest.mark.parametrize(
    "config_kwarg,header",
    [
        ("cache_control", CacheControlHeader(no_cache=True, documentation_only=True)),
        ("etag", ETag(value="1", documentation_only=True)),
    ],
)
def test_explicit_headers_documentation_only(config_kwarg: str, header: Header) -> None:
    @get(
        path="/test",
        **{config_kwarg: header},  # type: ignore[arg-type]
    )
    def my_handler() -> None:
        pass

    with create_test_client(my_handler) as client:
        response = client.get("/test")
        assert header.HEADER_NAME not in response.headers


@pytest.mark.parametrize(
    "config_kwarg,response_header,header",
    [
        (
            "cache_control",
            ResponseHeader(name=CacheControlHeader.HEADER_NAME, value="no-store"),
            CacheControlHeader(no_cache=True),
        ),
        ("etag", ResponseHeader(name=ETag.HEADER_NAME, value="1"), ETag(value="2")),
    ],
)
def test_explicit_headers_override_response_headers(
    config_kwarg: str, response_header: ResponseHeader, header: Header
) -> None:
    @get(
        path="/test",
        response_headers=[response_header],
        **{config_kwarg: header},  # type: ignore[arg-type]
    )
    def my_handler() -> None:
        pass

    app = Litestar(route_handlers=[my_handler])

    route_handler, _ = app.routes[0].route_handler_map[HttpMethod.GET]  # type: ignore[union-attr]
    resolved_headers = {header.name: header for header in route_handler.resolve_response_headers()}
    assert resolved_headers[header.HEADER_NAME].value == header.to_header()



from typing import Any
from unittest.mock import MagicMock

import pytest
from pytest_lazy_fixtures import lf

from litestar import Litestar, Request, get
from litestar.events.emitter import SimpleEventEmitter
from litestar.events.listener import EventListener, listener
from litestar.exceptions import ImproperlyConfiguredException
from litestar.status_codes import HTTP_200_OK
from litestar.testing import create_test_client
from litestar.types import AnyIOBackend


@pytest.fixture()
def mock() -> MagicMock:
    return MagicMock()


@pytest.fixture()
def sync_listener(mock: MagicMock) -> EventListener:
    @listener("test_event")
    def listener_fn(*args: Any, **kwargs: Any) -> None:
        mock(*args, **kwargs)

    return listener_fn


@pytest.fixture()
def async_listener(mock: MagicMock) -> EventListener:
    @listener("test_event")
    async def listener_fn(*args: Any, **kwargs: Any) -> None:
        mock(*args, **kwargs)

    return listener_fn


@pytest.mark.parametrize("event_listener", [lf("sync_listener"), lf("async_listener")])
def test_event_listener(mock: MagicMock, event_listener: EventListener, anyio_backend: AnyIOBackend) -> None:
    @get("/")
    def route_handler(request: Request[Any, Any, Any]) -> None:
        request.app.emit("test_event", "positional", keyword="keyword-value")

    with create_test_client(
        route_handlers=[route_handler], listeners=[event_listener], backend=anyio_backend
    ) as client:
        response = client.get("/")
        assert response.status_code == HTTP_200_OK

    mock.assert_called_with("positional", keyword="keyword-value")


async def test_shutdown_awaits_pending(async_listener: EventListener, mock: MagicMock) -> None:
    async with SimpleEventEmitter([async_listener]) as emitter:
        for _ in range(100):
            emitter.emit("test_event")

    assert mock.call_count == 100


def test_multiple_event_listeners(
    async_listener: EventListener, sync_listener: EventListener, mock: MagicMock, anyio_backend: AnyIOBackend
) -> None:
    @get("/")
    def route_handler(request: Request[Any, Any, Any]) -> None:
        request.app.emit("test_event")

    with create_test_client(
        route_handlers=[route_handler], listeners=[async_listener, sync_listener], backend=anyio_backend
    ) as client:
        response = client.get("/")
        assert response.status_code == HTTP_200_OK

    assert mock.call_count == 2


def test_multiple_event_ids(mock: MagicMock, anyio_backend: AnyIOBackend) -> None:
    @listener("test_event_1", "test_event_2")
    def event_handler() -> None:
        mock()

    @get("/{event_id:int}")
    def route_handler(request: Request[Any, Any, Any], event_id: int) -> None:
        request.app.emit(f"test_event_{event_id}")

    with create_test_client(route_handlers=[route_handler], listeners=[event_handler], backend=anyio_backend) as client:
        response_1 = client.get("/1")
        response_2 = client.get("/2")
        assert response_1.status_code == HTTP_200_OK
        assert response_2.status_code == HTTP_200_OK

    assert mock.call_count == 2


async def test_raises_when_decorator_called_without_callable() -> None:
    with pytest.raises(ImproperlyConfiguredException):
        listener("test_even")(True)  # type: ignore[arg-type]


async def test_raises_when_not_initialized() -> None:
    app = Litestar([])

    with pytest.raises(RuntimeError):
        app.emit("x")


async def test_raises_when_not_listener_are_registered_for_an_event_id(async_listener: EventListener) -> None:
    with create_test_client(route_handlers=[], listeners=[async_listener]) as client:
        with pytest.raises(ImproperlyConfiguredException):
            client.app.emit("x")


async def test_event_listener_raises_exception(async_listener: EventListener, mock: MagicMock) -> None:
    """Test that an event listener that raises an exception does not prevent other listeners from being called.

    https://github.com/litestar-org/litestar/issues/2809
    """

    error_mock = MagicMock()

    @listener("error_event")
    async def raising_listener(*args: Any, **kwargs: Any) -> None:
        error_mock()
        raise ValueError("test")

    @get("/error")
    def route_handler_1(request: Request[Any, Any, Any]) -> None:
        request.app.emit("error_event")

    @get("/no-error")
    def route_handler_2(request: Request[Any, Any, Any]) -> None:
        request.app.emit("test_event")

    with create_test_client(
        route_handlers=[route_handler_1, route_handler_2], listeners=[async_listener, raising_listener]
    ) as client:
        first_response = client.get("/error")
        second_response = client.get("/no-error")
        assert first_response.status_code == HTTP_200_OK
        assert second_response.status_code == HTTP_200_OK

    error_mock.assert_called()
    mock.assert_called()



from typing import TYPE_CHECKING, Any, Callable, Dict

import pytest

from litestar import (
    Controller,
    Litestar,
    Router,
    asgi,
    delete,
    get,
    patch,
    post,
    put,
    websocket,
)

if TYPE_CHECKING:
    from litestar import WebSocket
    from litestar.types import Receive, RouteHandlerType, Scope, Send


def regular_handler() -> None: ...


async def asgi_handler(scope: "Scope", receive: "Receive", send: "Send") -> None: ...


async def socket_handler(socket: "WebSocket") -> None: ...


@pytest.mark.parametrize(
    "decorator, handler",
    [
        (get, regular_handler),
        (post, regular_handler),
        (delete, regular_handler),
        (put, regular_handler),
        (patch, regular_handler),
        (asgi, asgi_handler),
        (websocket, socket_handler),
    ],
)
def test_opt_settings(decorator: "RouteHandlerType", handler: Callable) -> None:
    base_opt = {"base": 1, "kwarg_value": 0}
    result = decorator("/", opt=base_opt, kwarg_value=2)(handler)  # type: ignore[arg-type, call-arg]
    assert result.opt == {"base": 1, "kwarg_value": 2}


@pytest.mark.parametrize(
    "app_opt, router_opt, controller_opt, route_opt, expected_opt",
    [
        [
            {"app": "app"},
            {"router": "router"},
            {"controller": "controller"},
            {"route": "route"},
            {"app": "app", "router": "router", "controller": "controller", "route": "route"},
        ],
        [
            {"override": "app"},
            {"router": "router"},
            None,
            {"override": "route"},
            {"router": "router", "override": "route"},
        ],
        [None, None, None, None, {}],
    ],
)
def test_opt_resolution(
    app_opt: Dict[str, Any],
    router_opt: Dict[str, Any],
    controller_opt: Dict[str, Any],
    route_opt: Dict[str, Any],
    expected_opt: Dict[str, Any],
) -> None:
    class MyController(Controller):
        path = "/controller"
        opt = controller_opt

        @get(opt=route_opt)
        def handler(self) -> None: ...

    router = Router("/router", route_handlers=[MyController], opt=router_opt)
    app = Litestar(route_handlers=[router], opt=app_opt)
    assert (
        app.asgi_router.root_route_map_node.children["/router/controller"].asgi_handlers["GET"][1].opt == expected_opt
    )


def test_opt_not_affected_by_route_handler_copying() -> None:
    class MyController(Controller):
        path = "/controller"

        @get(opt={"route": "route"})
        def handler(self) -> None: ...

    @get("/fn_handler", opt={"fn_route": "fn_route"})
    def fn_handler() -> None: ...

    router = Router("/router", route_handlers=[MyController, fn_handler], opt={"router": "router"})
    another_router = Router("/another_router", route_handlers=[MyController, fn_handler])

    app = Litestar(route_handlers=[router, another_router])

    assert app.asgi_router.root_route_map_node.children["/router/controller"].asgi_handlers["GET"][1].opt == {
        "router": "router",
        "route": "route",
    }
    assert app.asgi_router.root_route_map_node.children["/router/fn_handler"].asgi_handlers["GET"][1].opt == {
        "router": "router",
        "fn_route": "fn_route",
    }

    assert app.asgi_router.root_route_map_node.children["/another_router/controller"].asgi_handlers["GET"][1].opt == {
        "route": "route",
    }

    assert app.asgi_router.root_route_map_node.children["/another_router/fn_handler"].asgi_handlers["GET"][1].opt == {
        "fn_route": "fn_route",
    }



from logging import INFO
from typing import TYPE_CHECKING, Any, Dict, Generator

import pytest
from structlog.testing import capture_logs
from typing_extensions import Annotated

from litestar import Response, get, post
from litestar.config.compression import CompressionConfig
from litestar.connection import Request
from litestar.datastructures import Cookie, UploadFile
from litestar.enums import RequestEncodingType
from litestar.exceptions import ImproperlyConfiguredException
from litestar.handlers import HTTPRouteHandler
from litestar.logging.config import LoggingConfig, StructLoggingConfig
from litestar.middleware import logging as middleware_logging
from litestar.middleware.logging import LoggingMiddlewareConfig
from litestar.params import Body
from litestar.status_codes import HTTP_200_OK, HTTP_201_CREATED
from litestar.testing import create_test_client
from tests.helpers import cleanup_logging_impl

if TYPE_CHECKING:
    from _pytest.logging import LogCaptureFixture
    from pytest import MonkeyPatch

    from litestar.middleware.session.server_side import ServerSideSessionConfig
    from litestar.types.callable_types import GetLogger


pytestmark = pytest.mark.usefixtures("reset_httpx_logging")


@pytest.fixture(autouse=True)
def cleanup_logging() -> Generator:
    with cleanup_logging_impl():
        yield


@pytest.fixture
def handler() -> HTTPRouteHandler:
    @get("/")
    def handler_fn() -> Response:
        return Response(
            content={"hello": "world"},
            headers={"token": "123", "regular": "abc"},
            cookies=[Cookie(key="first-cookie", value="abc"), Cookie(key="second-cookie", value="xxx")],
        )

    return handler_fn


def test_logging_middleware_config_validation() -> None:
    with pytest.raises(ImproperlyConfiguredException):
        LoggingMiddlewareConfig(response_log_fields=None)  # type: ignore[arg-type]

    with pytest.raises(ImproperlyConfiguredException):
        LoggingMiddlewareConfig(request_log_fields=None)  # type: ignore[arg-type]


def test_logging_middleware_regular_logger(
    get_logger: "GetLogger", caplog: "LogCaptureFixture", handler: HTTPRouteHandler
) -> None:
    with create_test_client(
        route_handlers=[handler], middleware=[LoggingMiddlewareConfig().middleware]
    ) as client, caplog.at_level(INFO):
        # Set cookies on the client to avoid warnings about per-request cookies.
        client.app.get_logger = get_logger
        client.cookies = {"request-cookie": "abc"}  # type: ignore[assignment]
        response = client.get("/", headers={"request-header": "1"})
        assert response.status_code == HTTP_200_OK
        assert len(caplog.messages) == 2

        assert (
            caplog.messages[0] == 'HTTP Request: path=/, method=GET, content_type=["",{}], '
            'headers={"host":"testserver.local","accept":"*/*","accept-encoding":"gzip, '
            'deflate, br","connection":"keep-alive","user-agent":"testclient",'
            '"request-header":"1","cookie":"request-cookie=abc"}, '
            'cookies={"request-cookie":"abc"}, query={}, path_params={}, body=None'
        )


def test_logging_middleware_struct_logger(handler: HTTPRouteHandler) -> None:
    with create_test_client(
        route_handlers=[handler],
        middleware=[LoggingMiddlewareConfig().middleware],
        logging_config=StructLoggingConfig(),
    ) as client, capture_logs() as cap_logs:
        # Set cookies on the client to avoid warnings about per-request cookies.
        client.cookies = {"request-cookie": "abc"}  # type: ignore[assignment]
        response = client.get("/", headers={"request-header": "1"})
        assert response.status_code == HTTP_200_OK
        assert len(cap_logs) == 2
        assert cap_logs[0] == {
            "path": "/",
            "method": "GET",
            "body": None,
            "content_type": ("", {}),
            "headers": {
                "host": "testserver.local",
                "accept": "*/*",
                "accept-encoding": "gzip, deflate, br",
                "connection": "keep-alive",
                "user-agent": "testclient",
                "request-header": "1",
                "cookie": "request-cookie=abc",
            },
            "cookies": {"request-cookie": "abc"},
            "query": {},
            "path_params": {},
            "event": "HTTP Request",
            "log_level": "info",
        }
        assert cap_logs[1] == {
            "status_code": 200,
            "cookies": {"first-cookie": "abc", "Path": "/", "SameSite": "lax", "second-cookie": "xxx"},
            "headers": {"token": "123", "regular": "abc", "content-length": "17", "content-type": "application/json"},
            "body": '{"hello":"world"}',
            "event": "HTTP Response",
            "log_level": "info",
        }


def test_logging_middleware_exclude_pattern(
    get_logger: "GetLogger", caplog: "LogCaptureFixture", handler: HTTPRouteHandler
) -> None:
    @get("/exclude")
    def handler2() -> None:
        return None

    config = LoggingMiddlewareConfig(exclude=["^/exclude"])
    with create_test_client(
        route_handlers=[handler, handler2], middleware=[config.middleware]
    ) as client, caplog.at_level(INFO):
        # Set cookies on the client to avoid warnings about per-request cookies.
        client.cookies = {"request-cookie": "abc"}  # type: ignore[assignment]
        client.app.get_logger = get_logger

        response = client.get("/exclude")
        assert response.status_code == HTTP_200_OK
        assert len(caplog.messages) == 0

        response = client.get("/")
        assert response.status_code == HTTP_200_OK
        assert len(caplog.messages) == 2


def test_logging_middleware_exclude_opt_key(
    get_logger: "GetLogger", caplog: "LogCaptureFixture", handler: HTTPRouteHandler
) -> None:
    @get("/exclude", skip_logging=True)
    def handler2() -> None:
        return None

    config = LoggingMiddlewareConfig(exclude_opt_key="skip_logging")
    with create_test_client(
        route_handlers=[handler, handler2], middleware=[config.middleware]
    ) as client, caplog.at_level(INFO):
        # Set cookies on the client to avoid warnings about per-request cookies.
        client.cookies = {"request-cookie": "abc"}  # type: ignore[assignment]
        client.app.get_logger = get_logger

        response = client.get("/exclude")
        assert response.status_code == HTTP_200_OK
        assert len(caplog.messages) == 0

        response = client.get("/")
        assert response.status_code == HTTP_200_OK
        assert len(caplog.messages) == 2


@pytest.mark.parametrize("include", [True, False])
def test_logging_middleware_compressed_response_body(
    get_logger: "GetLogger", include: bool, caplog: "LogCaptureFixture", handler: HTTPRouteHandler
) -> None:
    with create_test_client(
        route_handlers=[handler],
        compression_config=CompressionConfig(backend="gzip", minimum_size=1),
        middleware=[LoggingMiddlewareConfig(include_compressed_body=include).middleware],
    ) as client, caplog.at_level(INFO):
        # Set cookies on the client to avoid warnings about per-request cookies.
        client.cookies = {"request-cookie": "abc"}  # type: ignore[assignment]
        client.app.get_logger = get_logger
        response = client.get("/", headers={"request-header": "1"})
        assert response.status_code == HTTP_200_OK
        assert len(caplog.messages) == 2
        if include:
            assert "body=" in caplog.messages[1]
        else:
            assert "body=" not in caplog.messages[1]


def test_logging_middleware_post_body() -> None:
    @post("/")
    def post_handler(data: Dict[str, str]) -> Dict[str, str]:
        return data

    with create_test_client(
        route_handlers=[post_handler], middleware=[LoggingMiddlewareConfig().middleware], logging_config=LoggingConfig()
    ) as client:
        res = client.post("/", json={"foo": "bar"})
        assert res.status_code == 201
        assert res.json() == {"foo": "bar"}


async def test_logging_middleware_post_binary_file_without_structlog(monkeypatch: "MonkeyPatch") -> None:
    # https://github.com/litestar-org/litestar/issues/2529

    @post("/")
    async def post_handler(data: Annotated[UploadFile, Body(media_type=RequestEncodingType.MULTI_PART)]) -> str:
        content = await data.read()
        return f"{len(content)} bytes"

    # force LoggingConfig to not parse body data
    monkeypatch.setattr(middleware_logging, "structlog_installed", False)

    with create_test_client(
        route_handlers=[post_handler], middleware=[LoggingMiddlewareConfig().middleware], logging_config=LoggingConfig()
    ) as client:
        res = client.post("/", files={"foo": b"\xfa\xfb"})
        assert res.status_code == 201
        assert res.text == "2 bytes"


@pytest.mark.parametrize("logger_name", ("litestar", "other"))
def test_logging_messages_are_not_doubled(
    get_logger: "GetLogger", logger_name: str, caplog: "LogCaptureFixture"
) -> None:
    # https://github.com/litestar-org/litestar/issues/896

    @get("/")
    async def hello_world_handler() -> Dict[str, str]:
        return {"hello": "world"}

    logging_middleware_config = LoggingMiddlewareConfig(logger_name=logger_name)

    with create_test_client(
        hello_world_handler,
        logging_config=LoggingConfig(),
        middleware=[logging_middleware_config.middleware],
    ) as client, caplog.at_level(INFO):
        client.app.get_logger = get_logger
        response = client.get("/")
        assert response.status_code == HTTP_200_OK
        assert len(caplog.messages) == 2


def test_logging_middleware_log_fields(
    get_logger: "GetLogger", caplog: "LogCaptureFixture", handler: HTTPRouteHandler
) -> None:
    with create_test_client(
        route_handlers=[handler],
        middleware=[
            LoggingMiddlewareConfig(response_log_fields=["status_code"], request_log_fields=["path"]).middleware
        ],
    ) as client, caplog.at_level(INFO):
        # Set cookies on the client to avoid warnings about per-request cookies.
        client.app.get_logger = get_logger
        client.cookies = {"request-cookie": "abc"}  # type: ignore[assignment]
        response = client.get("/", headers={"request-header": "1"})
        assert response.status_code == HTTP_200_OK
        assert len(caplog.messages) == 2

        assert caplog.messages[0] == "HTTP Request: path=/"
        assert caplog.messages[1] == "HTTP Response: status_code=200"


def test_logging_middleware_with_session_middleware(session_backend_config_memory: "ServerSideSessionConfig") -> None:
    # https://github.com/litestar-org/litestar/issues/1228

    @post("/")
    async def set_session(request: Request) -> None:
        request.set_session({"hello": "world"})

    @get("/")
    async def get_session() -> None:
        pass

    logging_middleware_config = LoggingMiddlewareConfig()

    with create_test_client(
        [set_session, get_session],
        logging_config=LoggingConfig(),
        middleware=[logging_middleware_config.middleware, session_backend_config_memory.middleware],
    ) as client:
        response = client.post("/")
        assert response.status_code == HTTP_201_CREATED
        assert "session" in client.cookies
        assert client.cookies["session"] != "*****"
        session_id = client.cookies["session"]

        response = client.get("/")
        assert response.status_code == HTTP_200_OK
        assert "session" in client.cookies
        assert client.cookies["session"] == session_id


def test_structlog_invalid_request_body_handled() -> None:
    # https://github.com/litestar-org/litestar/issues/3063
    @post("/")
    async def hello_world(data: Dict[str, Any]) -> Dict[str, Any]:
        return data

    with create_test_client(
        route_handlers=[hello_world],
        logging_config=StructLoggingConfig(log_exceptions="always"),
        middleware=[LoggingMiddlewareConfig().middleware],
    ) as client:
        assert client.post("/", headers={"Content-Type": "application/json"}, content=b'{"a": "b",}').status_code == 400



from __future__ import annotations

import typing
from dataclasses import replace
from decimal import Decimal
from enum import Enum
from typing import TYPE_CHECKING, Any

import msgspec
from polyfactory.exceptions import ParameterException
from polyfactory.factories import DataclassFactory
from polyfactory.field_meta import FieldMeta, Null
from polyfactory.utils.helpers import unwrap_annotation
from polyfactory.utils.predicates import is_union
from typing_extensions import get_args

from litestar.contrib.pydantic.utils import is_pydantic_model_instance
from litestar.openapi.spec import Example
from litestar.types import Empty

if TYPE_CHECKING:
    from litestar.typing import FieldDefinition


class ExampleFactory(DataclassFactory[Example]):
    __model__ = Example
    __random_seed__ = 10


def _normalize_example_value(value: Any) -> Any:
    """Normalize the example value to make it look a bit prettier."""
    # if UnsetType is part of the union, then it might get chosen as the value
    # but that will not be properly serialized by msgspec unless it is for a field
    # in a msgspec Struct
    if is_union(value):
        args = list(get_args(value))
        try:
            args.remove(msgspec.UnsetType)
            value = typing.Union[tuple(args)]  # pyright: ignore
        except ValueError:
            # UnsetType not part of the Union
            pass

    value = unwrap_annotation(annotation=value, random=ExampleFactory.__random__)
    if isinstance(value, (Decimal, float)):
        value = round(float(value), 2)
    if isinstance(value, Enum):
        value = value.value
    if is_pydantic_model_instance(value):
        from litestar.contrib.pydantic import _model_dump

        value = _model_dump(value)
    if isinstance(value, (list, set)):
        value = [_normalize_example_value(v) for v in value]
    if isinstance(value, dict):
        for k, v in value.items():
            value[k] = _normalize_example_value(v)
    return value


def _create_field_meta(field: FieldDefinition) -> FieldMeta:
    return FieldMeta.from_type(
        annotation=field.annotation,
        default=field.default if field.default is not Empty else Null,
        name=field.name,
        random=ExampleFactory.__random__,
    )


def create_examples_for_field(field: FieldDefinition) -> list[Example]:
    """Create an OpenAPI Example instance.

    Args:
        field: A signature field.

    Returns:
        A list including a single example.
    """
    try:
        field_meta = _create_field_meta(replace(field, annotation=_normalize_example_value(field.annotation)))
        value = ExampleFactory.get_field_value(field_meta)
        return [Example(description=f"Example {field.name} value", value=value)]
    except ParameterException:
        return []



"""A repository implementation for tests.

Uses a `dict` for storage.
"""

from __future__ import annotations

from datetime import datetime, timezone, tzinfo
from typing import TYPE_CHECKING, Generic, Protocol, TypeVar
from uuid import uuid4

from litestar.repository import AbstractAsyncRepository, AbstractSyncRepository, FilterTypes
from litestar.repository.exceptions import ConflictError, RepositoryError

if TYPE_CHECKING:
    from collections.abc import Callable, Hashable, Iterable, MutableMapping
    from typing import Any

__all__ = ("GenericAsyncMockRepository", "GenericSyncMockRepository")


class HasID(Protocol):
    id: Any


ModelT = TypeVar("ModelT", bound="HasID")
AsyncMockRepoT = TypeVar("AsyncMockRepoT", bound="GenericAsyncMockRepository")
SyncMockRepoT = TypeVar("SyncMockRepoT", bound="GenericSyncMockRepository")


class GenericAsyncMockRepository(AbstractAsyncRepository[ModelT], Generic[ModelT]):
    """A repository implementation for tests.

    Uses a :class:`dict` for storage.
    """

    collection: MutableMapping[Hashable, ModelT]
    model_type: type[ModelT]
    match_fields: list[str] | str | None = None

    _model_has_created_at: bool
    _model_has_updated_at: bool

    def __init__(
        self, id_factory: Callable[[], Any] = uuid4, tz: tzinfo = timezone.utc, allow_ids_on_add: bool = False, **_: Any
    ) -> None:
        super().__init__()
        self._id_factory = id_factory
        self.tz = tz
        self.allow_ids_on_add = allow_ids_on_add

    @classmethod
    def __class_getitem__(cls: type[AsyncMockRepoT], item: type[ModelT]) -> type[AsyncMockRepoT]:
        """Add collection to ``_collections`` for the type.

        Args:
            item: The type that the class has been parametrized with.
        """
        return type(  # pyright:ignore
            f"{cls.__name__}[{item.__name__}]",
            (cls,),
            {
                "collection": {},
                "model_type": item,
                "_model_has_created_at": hasattr(item, "created_at"),
                "_model_has_updated_at": hasattr(item, "updated_at"),
            },
        )

    def _find_or_raise_not_found(self, item_id: Any) -> ModelT:
        return self.check_not_found(self.collection.get(item_id))

    def _find_or_none(self, item_id: Any) -> ModelT | None:
        return self.collection.get(item_id)

    def _now(self) -> datetime:
        return datetime.now(tz=self.tz).replace(tzinfo=None)

    def _update_audit_attributes(self, data: ModelT, now: datetime | None = None, do_created: bool = False) -> ModelT:
        now = now or self._now()
        if self._model_has_updated_at:
            data.updated_at = now  # type:ignore[attr-defined]
            if do_created:
                data.created_at = now  # type:ignore[attr-defined]
        return data

    async def add(self, data: ModelT) -> ModelT:
        """Add ``data`` to the collection.

        Args:
            data: Instance to be added to the collection.

        Returns:
            The added instance.
        """
        if self.allow_ids_on_add is False and self.get_id_attribute_value(data) is not None:
            raise ConflictError("`add()` received identified item.")
        self._update_audit_attributes(data, do_created=True)
        if self.allow_ids_on_add is False:
            id_ = self._id_factory()
            self.set_id_attribute_value(id_, data)
        self.collection[data.id] = data
        return data

    async def add_many(self, data: Iterable[ModelT]) -> list[ModelT]:
        """Add multiple ``data`` to the collection.

        Args:
            data: Instance to be added to the collection.

        Returns:
            The added instance.
        """
        now = self._now()
        for data_row in data:
            if self.allow_ids_on_add is False and self.get_id_attribute_value(data_row) is not None:
                raise ConflictError("`add()` received identified item.")

            self._update_audit_attributes(data_row, do_created=True, now=now)
            if self.allow_ids_on_add is False:
                id_ = self._id_factory()
                self.set_id_attribute_value(id_, data_row)
                self.collection[data_row.id] = data_row
        return list(data)

    async def delete(self, item_id: Any) -> ModelT:
        """Delete instance identified by ``item_id``.

        Args:
            item_id: Identifier of instance to be deleted.

        Returns:
            The deleted instance.

        Raises:
            NotFoundError: If no instance found identified by ``item_id``.
        """
        try:
            return self._find_or_raise_not_found(item_id)
        finally:
            del self.collection[item_id]

    async def delete_many(self, item_ids: list[Any]) -> list[ModelT]:
        """Delete instances identified by list of identifiers ``item_ids``.

        Args:
            item_ids: list of identifiers of instances to be deleted.

        Returns:
            The deleted instances.

        """
        instances: list[ModelT] = []
        for item_id in item_ids:
            obj = await self.get_one_or_none(**{self.id_attribute: item_id})
            if obj:
                obj = await self.delete(obj.id)
                instances.append(obj)
        return instances

    async def exists(self, *filters: FilterTypes, **kwargs: Any) -> bool:
        """Return true if the object specified by ``kwargs`` exists.

        Args:
            *filters: Types for specific filtering operations.
            **kwargs: Identifier of the instance to be retrieved.

        Returns:
            True if the instance was found.  False if not found..

        """
        existing = await self.count(*filters, **kwargs)
        return bool(existing)

    async def get(self, item_id: Any, **kwargs: Any) -> ModelT:
        """Get instance identified by ``item_id``.

        Args:
            item_id: Identifier of the instance to be retrieved.
            **kwargs: additional arguments

        Returns:
            The retrieved instance.

        Raises:
            NotFoundError: If no instance found identified by ``item_id``.
        """
        return self._find_or_raise_not_found(item_id)

    async def get_or_create(self, match_fields: list[str] | str | None = None, **kwargs: Any) -> tuple[ModelT, bool]:
        """Get instance identified by ``kwargs`` or create if it doesn't exist.

        Args:
            match_fields: a list of keys to use to match the existing model.  When empty, all fields are matched.
            **kwargs: Identifier of the instance to be retrieved.

        Returns:
            a tuple that includes the instance and whether it needed to be created.

        """
        match_fields = match_fields or self.match_fields
        if isinstance(match_fields, str):
            match_fields = [match_fields]
        if match_fields:
            match_filter = {
                field_name: field_value
                for field_name in match_fields
                if (field_value := kwargs.get(field_name)) is not None
            }
        else:
            match_filter = kwargs
        existing = await self.get_one_or_none(**match_filter)
        if existing:
            for field_name, new_field_value in kwargs.items():
                field = getattr(existing, field_name, None)
                if field and field != new_field_value:
                    setattr(existing, field_name, new_field_value)

            return existing, False
        return await self.add(self.model_type(**kwargs)), True  # pyright: ignore[reportGeneralTypeIssues]

    async def get_one(self, **kwargs: Any) -> ModelT:
        """Get instance identified by query filters.

        Args:
            **kwargs: Instance attribute value filters.

        Returns:
            The retrieved instance or None

        Raises:
            NotFoundError: If no instance found identified by ``kwargs``.
        """
        data = await self.list(**kwargs)
        return self.check_not_found(data[0] if data else None)

    async def get_one_or_none(self, **kwargs: Any) -> ModelT | None:
        """Get instance identified by query filters or None if not found.

        Args:
            **kwargs: Instance attribute value filters.

        Returns:
            The retrieved instance or None
        """
        data = await self.list(**kwargs)
        return data[0] if data else None

    async def count(self, *filters: FilterTypes, **kwargs: Any) -> int:
        """Count of rows returned by query.

        Args:
            *filters: Types for specific filtering operations.
            **kwargs: Instance attribute value filters.

        Returns:
            Count of instances in collection, ignoring pagination.
        """
        return len(await self.list(*filters, **kwargs))

    async def update(self, data: ModelT) -> ModelT:
        """Update instance with the attribute values present on ``data``.

        Args:
            data: An instance that should have a value for :attr:`id_attribute <AsyncGenericMockRepository.id_attribute>` that exists in the
                collection.

        Returns:
            The updated instance.

        Raises:
            NotFoundError: If no instance found with same identifier as ``data``.
        """
        item = self._find_or_raise_not_found(self.get_id_attribute_value(data))
        self._update_audit_attributes(data, do_created=False)
        for key, val in model_items(data):
            setattr(item, key, val)
        return item

    async def update_many(self, data: list[ModelT]) -> list[ModelT]:
        """Update instances with the attribute values present on ``data``.

        Args:
            data: A list of instances that should have a value for :attr:`id_attribute <AsyncGenericMockRepository.id_attribute>`
                that exists in the collection.

        Returns:
            The updated instances.

        Raises:
            NotFoundError: If no instance found with same identifier as ``data``.
        """
        items = [self._find_or_raise_not_found(self.get_id_attribute_value(row)) for row in data]
        now = self._now()
        for item in items:
            self._update_audit_attributes(item, do_created=False, now=now)
            for key, val in model_items(item):
                setattr(item, key, val)
        return items

    async def upsert(self, data: ModelT) -> ModelT:
        """Update or create instance.

        Updates instance with the attribute values present on ``data``, or creates a new instance if
        one doesn't exist.

        Args:
            data: Instance to update existing, or be created. Identifier used to determine if an
                existing instance exists is the value of an attribute on `data` named as value of
                :attr:`id_attribute <AsyncGenericMockRepository.id_attribute>`.

        Returns:
            The updated or created instance.

        Raises:
            NotFoundError: If no instance found with same identifier as ``data``.
        """
        item_id = self.get_id_attribute_value(data)
        if item_id in self.collection:
            return await self.update(data)
        return await self.add(data)

    async def upsert_many(self, data: list[ModelT]) -> list[ModelT]:
        """Update or create multiple instance.

        Update instance with the attribute values present on ``data``, or create a new instance if
        one doesn't exist.

        Args:
            data: List of instances to update existing, or be created. Identifier used to determine if an
                existing instance exists is the value of an attribute on `data` named as value of
                :attr:`id_attribute <AsyncGenericMockRepository.id_attribute>`.

        Returns:
            The updated or created instances.
        """
        data_to_update = [row for row in data if self._find_or_none(self.get_id_attribute_value(row)) is not None]
        data_to_add = [row for row in data if self._find_or_none(self.get_id_attribute_value(row)) is None]

        updated_items = await self.update_many(data_to_update)
        added_items = await self.add_many(data_to_add)
        return updated_items + added_items

    async def list_and_count(
        self,
        *filters: FilterTypes,
        **kwargs: Any,
    ) -> tuple[list[ModelT], int]:
        """Get a list of instances, optionally filtered with a total row count.

        Args:
            *filters: Types for specific filtering operations.
            **kwargs: Instance attribute value filters.

        Returns:
            List of instances, and count of records returned by query, ignoring pagination.
        """
        return await self.list(*filters, **kwargs), await self.count(*filters, **kwargs)

    async def list(self, *filters: FilterTypes, **kwargs: Any) -> list[ModelT]:
        """Get a list of instances, optionally filtered.

        Args:
            *filters: Types for specific filtering operations.
            **kwargs: Instance attribute value filters.

        Returns:
            The list of instances, after filtering applied.
        """
        return list(self.filter_collection_by_kwargs(self.collection, **kwargs).values())

    def filter_collection_by_kwargs(  # type:ignore[override]
        self, collection: MutableMapping[Hashable, ModelT], /, **kwargs: Any
    ) -> MutableMapping[Hashable, ModelT]:
        """Filter the collection by kwargs.

        Args:
            collection: set of objects to filter
            **kwargs: key/value pairs such that objects remaining in the collection after filtering
                have the property that their attribute named ``key`` has value equal to ``value``.
        """
        new_collection: dict[Hashable, ModelT] = {}
        for item in self.collection.values():
            try:
                if all(getattr(item, name) == value for name, value in kwargs.items()):
                    new_collection[item.id] = item
            except AttributeError as orig:
                raise RepositoryError from orig
        return new_collection

    @classmethod
    def seed_collection(cls, instances: Iterable[ModelT]) -> None:
        """Seed the collection for repository type.

        Args:
            instances: the instances to be added to the collection.
        """
        for instance in instances:
            cls.collection[cls.get_id_attribute_value(instance)] = instance

    @classmethod
    def clear_collection(cls) -> None:
        """Empty the collection for repository type."""
        cls.collection = {}


class GenericSyncMockRepository(AbstractSyncRepository[ModelT], Generic[ModelT]):
    """A repository implementation for tests.

    Uses a :class:`dict` for storage.
    """

    collection: MutableMapping[Hashable, ModelT]
    model_type: type[ModelT]
    match_fields: list[str] | str | None = None

    _model_has_created_at: bool
    _model_has_updated_at: bool

    def __init__(
        self,
        id_factory: Callable[[], Any] = uuid4,
        tz: tzinfo = timezone.utc,
        allow_ids_on_add: bool = False,
        **_: Any,
    ) -> None:
        super().__init__()
        self._id_factory = id_factory
        self.tz = tz
        self.allow_ids_on_add = allow_ids_on_add

    @classmethod
    def __class_getitem__(cls: type[SyncMockRepoT], item: type[ModelT]) -> type[SyncMockRepoT]:
        """Add collection to ``_collections`` for the type.

        Args:
            item: The type that the class has been parametrized with.
        """
        return type(  # pyright:ignore
            f"{cls.__name__}[{item.__name__}]",
            (cls,),
            {
                "collection": {},
                "model_type": item,
                "_model_has_created_at": hasattr(item, "created_at"),
                "_model_has_updated_at": hasattr(item, "updated_at"),
            },
        )

    def _find_or_raise_not_found(self, item_id: Any) -> ModelT:
        return self.check_not_found(self.collection.get(item_id))

    def _find_or_none(self, item_id: Any) -> ModelT | None:
        return self.collection.get(item_id)

    def _now(self) -> datetime:
        return datetime.now(tz=self.tz).replace(tzinfo=None)

    def _update_audit_attributes(self, data: ModelT, now: datetime | None = None, do_created: bool = False) -> ModelT:
        now = now or self._now()
        if self._model_has_updated_at:
            data.updated_at = now  # type:ignore[attr-defined]
            if do_created:
                data.created_at = now  # type:ignore[attr-defined]
        return data

    def add(self, data: ModelT) -> ModelT:
        """Add ``data`` to the collection.

        Args:
            data: Instance to be added to the collection.

        Returns:
            The added instance.
        """
        if self.allow_ids_on_add is False and self.get_id_attribute_value(data) is not None:
            raise ConflictError("`add()` received identified item.")
        self._update_audit_attributes(data, do_created=True)
        if self.allow_ids_on_add is False:
            id_ = self._id_factory()
            self.set_id_attribute_value(id_, data)
        self.collection[data.id] = data
        return data

    def add_many(self, data: Iterable[ModelT]) -> list[ModelT]:
        """Add multiple ``data`` to the collection.

        Args:
            data: Instance to be added to the collection.

        Returns:
            The added instance.
        """
        now = self._now()
        for data_row in data:
            if self.allow_ids_on_add is False and self.get_id_attribute_value(data_row) is not None:
                raise ConflictError("`add()` received identified item.")

            self._update_audit_attributes(data_row, do_created=True, now=now)
            if self.allow_ids_on_add is False:
                id_ = self._id_factory()
                self.set_id_attribute_value(id_, data_row)
                self.collection[data_row.id] = data_row
        return list(data)

    def delete(self, item_id: Any) -> ModelT:
        """Delete instance identified by ``item_id``.

        Args:
            item_id: Identifier of instance to be deleted.

        Returns:
            The deleted instance.

        Raises:
            NotFoundError: If no instance found identified by ``item_id``.
        """
        try:
            return self._find_or_raise_not_found(item_id)
        finally:
            del self.collection[item_id]

    def delete_many(self, item_ids: list[Any]) -> list[ModelT]:
        """Delete instances identified by list of identifiers ``item_ids``.

        Args:
            item_ids: list of identifiers of instances to be deleted.

        Returns:
            The deleted instances.

        """
        instances: list[ModelT] = []
        for item_id in item_ids:
            if obj := self.get_one_or_none(**{self.id_attribute: item_id}):
                obj = self.delete(obj.id)
                instances.append(obj)
        return instances

    def exists(self, *filters: FilterTypes, **kwargs: Any) -> bool:
        """Return true if the object specified by ``kwargs`` exists.

        Args:
            *filters: Types for specific filtering operations.
            **kwargs: Identifier of the instance to be retrieved.

        Returns:
            True if the instance was found.  False if not found..

        """
        existing = self.count(*filters, **kwargs)
        return bool(existing)

    def get(self, item_id: Any, **kwargs: Any) -> ModelT:
        """Get instance identified by ``item_id``.

        Args:
            item_id: Identifier of the instance to be retrieved.
            **kwargs: additional arguments

        Returns:
            The retrieved instance.

        Raises:
            NotFoundError: If no instance found identified by ``item_id``.
        """
        return self._find_or_raise_not_found(item_id)

    def get_or_create(self, match_fields: list[str] | str | None = None, **kwargs: Any) -> tuple[ModelT, bool]:
        """Get instance identified by ``kwargs`` or create if it doesn't exist.

        Args:
            match_fields: a list of keys to use to match the existing model.  When empty, all fields are matched.
            **kwargs: Identifier of the instance to be retrieved.

        Returns:
            a tuple that includes the instance and whether it needed to be created.

        """
        match_fields = match_fields or self.match_fields
        if isinstance(match_fields, str):
            match_fields = [match_fields]
        if match_fields:
            match_filter = {
                field_name: field_value
                for field_name in match_fields
                if (field_value := kwargs.get(field_name)) is not None
            }
        else:
            match_filter = kwargs
        if existing := self.get_one_or_none(**match_filter):
            for field_name, new_field_value in kwargs.items():
                field = getattr(existing, field_name, None)
                if field and field != new_field_value:
                    setattr(existing, field_name, new_field_value)

            return existing, False
        return self.add(self.model_type(**kwargs)), True  # pyright: ignore[reportGeneralTypeIssues]

    def get_one(self, **kwargs: Any) -> ModelT:
        """Get instance identified by query filters.

        Args:
            **kwargs: Instance attribute value filters.

        Returns:
            The retrieved instance or None

        Raises:
            NotFoundError: If no instance found identified by ``kwargs``.
        """
        data = self.list(**kwargs)
        return self.check_not_found(data[0] if data else None)

    def get_one_or_none(self, **kwargs: Any) -> ModelT | None:
        """Get instance identified by query filters or None if not found.

        Args:
            **kwargs: Instance attribute value filters.

        Returns:
            The retrieved instance or None
        """
        data = self.list(**kwargs)
        return data[0] if data else None

    def count(self, *filters: FilterTypes, **kwargs: Any) -> int:
        """Count of rows returned by query.

        Args:
            *filters: Types for specific filtering operations.
            **kwargs: Instance attribute value filters.

        Returns:
            Count of instances in collection, ignoring pagination.
        """
        return len(self.list(*filters, **kwargs))

    def update(self, data: ModelT) -> ModelT:
        """Update instance with the attribute values present on ``data``.

        Args:
            data: An instance that should have a value for :attr:`id_attribute <AsyncGenericMockRepository.id_attribute>` that exists in the
                collection.

        Returns:
            The updated instance.

        Raises:
            NotFoundError: If no instance found with same identifier as ``data``.
        """
        item = self._find_or_raise_not_found(self.get_id_attribute_value(data))
        self._update_audit_attributes(data, do_created=False)
        for key, val in model_items(data):
            setattr(item, key, val)
        return item

    def update_many(self, data: list[ModelT]) -> list[ModelT]:
        """Update instances with the attribute values present on ``data``.

        Args:
            data: A list of instances that should have a value for :attr:`id_attribute <AsyncGenericMockRepository.id_attribute>`
                that exists in the collection.

        Returns:
            The updated instances.

        Raises:
            NotFoundError: If no instance found with same identifier as ``data``.
        """
        items = [self._find_or_raise_not_found(self.get_id_attribute_value(row)) for row in data]
        now = self._now()
        for item in items:
            self._update_audit_attributes(item, do_created=False, now=now)
            for key, val in model_items(item):
                setattr(item, key, val)
        return items

    def upsert(self, data: ModelT) -> ModelT:
        """Update or create instance.

        Updates instance with the attribute values present on ``data``, or creates a new instance if
        one doesn't exist.

        Args:
            data: Instance to update existing, or be created. Identifier used to determine if an
                existing instance exists is the value of an attribute on `data` named as value of
                :attr:`id_attribute <AsyncGenericMockRepository.id_attribute>`.

        Returns:
            The updated or created instance.

        Raises:
            NotFoundError: If no instance found with same identifier as ``data``.
        """
        item_id = self.get_id_attribute_value(data)
        return self.update(data) if item_id in self.collection else self.add(data)

    def upsert_many(self, data: list[ModelT]) -> list[ModelT]:
        """Update or create multiple instance.

        Update instance with the attribute values present on ``data``, or create a new instance if
        one doesn't exist.

        Args:
            data: List of instances to update existing, or be created. Identifier used to determine if an
                existing instance exists is the value of an attribute on `data` named as value of
                :attr:`id_attribute <AsyncGenericMockRepository.id_attribute>`.

        Returns:
            The updated or created instances.
        """
        data_to_update = [row for row in data if self._find_or_none(self.get_id_attribute_value(row)) is not None]
        data_to_add = [row for row in data if self._find_or_none(self.get_id_attribute_value(row)) is None]

        updated_items = self.update_many(data_to_update)
        added_items = self.add_many(data_to_add)
        return updated_items + added_items

    def list_and_count(
        self,
        *filters: FilterTypes,
        **kwargs: Any,
    ) -> tuple[list[ModelT], int]:
        """Get a list of instances, optionally filtered with a total row count.

        Args:
            *filters: Types for specific filtering operations.
            **kwargs: Instance attribute value filters.

        Returns:
            List of instances, and count of records returned by query, ignoring pagination.
        """
        return self.list(*filters, **kwargs), self.count(*filters, **kwargs)

    def list(self, *filters: FilterTypes, **kwargs: Any) -> list[ModelT]:
        """Get a list of instances, optionally filtered.

        Args:
            *filters: Types for specific filtering operations.
            **kwargs: Instance attribute value filters.

        Returns:
            The list of instances, after filtering applied.
        """
        return list(self.filter_collection_by_kwargs(self.collection, **kwargs).values())

    def filter_collection_by_kwargs(  # type:ignore[override]
        self, collection: MutableMapping[Hashable, ModelT], /, **kwargs: Any
    ) -> MutableMapping[Hashable, ModelT]:
        """Filter the collection by kwargs.

        Args:
            collection: set of objects to filter
            **kwargs: key/value pairs such that objects remaining in the collection after filtering
                have the property that their attribute named ``key`` has value equal to ``value``.
        """
        new_collection: dict[Hashable, ModelT] = {}
        for item in self.collection.values():
            try:
                if all(getattr(item, name) == value for name, value in kwargs.items()):
                    new_collection[item.id] = item
            except AttributeError as orig:
                raise RepositoryError from orig
        return new_collection

    @classmethod
    def seed_collection(cls, instances: Iterable[ModelT]) -> None:
        """Seed the collection for repository type.

        Args:
            instances: the instances to be added to the collection.
        """
        for instance in instances:
            cls.collection[cls.get_id_attribute_value(instance)] = instance

    @classmethod
    def clear_collection(cls) -> None:
        """Empty the collection for repository type."""
        cls.collection = {}


def model_items(model: Any) -> list[tuple[str, Any]]:
    return [(k, v) for k, v in model.__dict__.items() if not k.startswith("_")]



from __future__ import annotations

from gzip import GzipFile
from typing import TYPE_CHECKING, Literal

from litestar.enums import CompressionEncoding
from litestar.middleware.compression.facade import CompressionFacade

if TYPE_CHECKING:
    from io import BytesIO

    from litestar.config.compression import CompressionConfig


class GzipCompression(CompressionFacade):
    __slots__ = ("compressor", "buffer", "compression_encoding")

    encoding = CompressionEncoding.GZIP

    def __init__(
        self, buffer: BytesIO, compression_encoding: Literal[CompressionEncoding.GZIP] | str, config: CompressionConfig
    ) -> None:
        self.buffer = buffer
        self.compression_encoding = compression_encoding
        self.compressor = GzipFile(mode="wb", fileobj=buffer, compresslevel=config.gzip_compress_level)

    def write(self, body: bytes) -> None:
        self.compressor.write(body)
        self.compressor.flush()

    def close(self) -> None:
        self.compressor.close()



from __future__ import annotations

from contextlib import contextmanager
from http.cookiejar import CookieJar
from typing import TYPE_CHECKING, Any, Generator, Generic, Mapping, TypeVar, cast
from warnings import warn

from anyio.from_thread import BlockingPortal, start_blocking_portal
from httpx import Cookies, Request, Response

from litestar import Litestar
from litestar.connection import ASGIConnection
from litestar.datastructures import MutableScopeHeaders
from litestar.enums import ScopeType
from litestar.exceptions import (
    ImproperlyConfiguredException,
)
from litestar.types import AnyIOBackend, ASGIApp, HTTPResponseStartEvent
from litestar.utils.scope.state import ScopeState

if TYPE_CHECKING:
    from httpx._types import CookieTypes

    from litestar.middleware.session.base import BaseBackendConfig, BaseSessionBackend
    from litestar.types.asgi_types import HTTPScope, Receive, Scope, Send

T = TypeVar("T", bound=ASGIApp)


def fake_http_send_message(headers: MutableScopeHeaders) -> HTTPResponseStartEvent:
    headers.setdefault("content-type", "application/text")
    return HTTPResponseStartEvent(type="http.response.start", status=200, headers=headers.headers)


def fake_asgi_connection(app: ASGIApp, cookies: dict[str, str]) -> ASGIConnection[Any, Any, Any, Any]:
    scope: HTTPScope = {
        "type": ScopeType.HTTP,
        "path": "/",
        "raw_path": b"/",
        "root_path": "",
        "scheme": "http",
        "query_string": b"",
        "client": ("testclient", 50000),
        "server": ("testserver", 80),
        "headers": [],
        "method": "GET",
        "http_version": "1.1",
        "extensions": {"http.response.template": {}},
        "app": app,  # type: ignore[typeddict-item]
        "state": {},
        "path_params": {},
        "route_handler": None,  # type: ignore[typeddict-item]
        "asgi": {"version": "3.0", "spec_version": "2.1"},
        "auth": None,
        "session": None,
        "user": None,
    }
    ScopeState.from_scope(scope).cookies = cookies
    return ASGIConnection[Any, Any, Any, Any](scope=scope)


def _wrap_app_to_add_state(app: ASGIApp) -> ASGIApp:
    """Wrap an ASGI app to add state to the scope.

    Litestar depends on `state` being present in the ASGI connection scope. Scope state is optional in the ASGI spec,
    however, the Litestar app always ensures it is present so that it can be depended on internally.

    When the ASGI app that is passed to the test client is _not_ a Litestar app, we need to add
    state to the scope, because httpx does not do this for us.

    This assists us in testing Litestar components that rely on state being present in the scope, without having
    to create a Litestar app for every test case.

    Args:
        app: The ASGI app to wrap.

    Returns:
        The wrapped ASGI app.
    """

    async def wrapped(scope: Scope, receive: Receive, send: Send) -> None:
        scope["state"] = {}
        await app(scope, receive, send)

    return wrapped


class BaseTestClient(Generic[T]):
    __test__ = False
    blocking_portal: BlockingPortal

    __slots__ = (
        "app",
        "base_url",
        "backend",
        "backend_options",
        "session_config",
        "_session_backend",
        "cookies",
    )

    def __init__(
        self,
        app: T,
        base_url: str = "http://testserver.local",
        backend: AnyIOBackend = "asyncio",
        backend_options: Mapping[str, Any] | None = None,
        session_config: BaseBackendConfig | None = None,
        cookies: CookieTypes | None = None,
    ) -> None:
        if "." not in base_url:
            warn(
                f"The base_url {base_url!r} might cause issues. Try adding a domain name such as .local: "
                f"'{base_url}.local'",
                UserWarning,
                stacklevel=1,
            )

        self._session_backend: BaseSessionBackend | None = None
        if session_config:
            self._session_backend = session_config._backend_class(config=session_config)

        if not isinstance(app, Litestar):
            app = _wrap_app_to_add_state(app)  # type: ignore[assignment]

        self.app = cast("T", app)  # type: ignore[redundant-cast]  # pyright needs this

        self.base_url = base_url
        self.backend = backend
        self.backend_options = backend_options
        self.cookies = cookies

    @property
    def session_backend(self) -> BaseSessionBackend[Any]:
        if not self._session_backend:
            raise ImproperlyConfiguredException(
                "Session has not been initialized for this TestClient instance. You can"
                "do so by passing a configuration object to TestClient: TestClient(app=app, session_config=...)"
            )
        return self._session_backend

    @contextmanager
    def portal(self) -> Generator[BlockingPortal, None, None]:
        """Get a BlockingPortal.

        Returns:
            A contextmanager for a BlockingPortal.
        """
        if hasattr(self, "blocking_portal"):
            yield self.blocking_portal
        else:
            with start_blocking_portal(
                backend=self.backend, backend_options=dict(self.backend_options or {})
            ) as portal:
                yield portal

    async def _set_session_data(self, data: dict[str, Any]) -> None:
        mutable_headers = MutableScopeHeaders()
        connection = fake_asgi_connection(
            app=self.app,
            cookies=dict(self.cookies),  # type: ignore[arg-type]
        )
        session_id = self.session_backend.get_session_id(connection)
        connection._connection_state.session_id = session_id  # pyright: ignore [reportGeneralTypeIssues]
        await self.session_backend.store_in_message(
            scope_session=data, message=fake_http_send_message(mutable_headers), connection=connection
        )
        response = Response(200, request=Request("GET", self.base_url), headers=mutable_headers.headers)

        cookies = Cookies(CookieJar())
        cookies.extract_cookies(response)
        self.cookies.update(cookies)  # type: ignore[union-attr]

    async def _get_session_data(self) -> dict[str, Any]:
        return await self.session_backend.load_from_connection(
            connection=fake_asgi_connection(
                app=self.app,
                cookies=dict(self.cookies),  # type: ignore[arg-type]
            ),
        )



from docs.examples.application_state.using_custom_state import app

from litestar.status_codes import HTTP_200_OK
from litestar.testing import TestClient


def test_using_custom_state_example() -> None:
    with TestClient(app) as client:
        response = client.get("/")
        assert response.status_code == HTTP_200_OK
        assert response.json() == {"count": 1}



from __future__ import annotations

import dataclasses
from dataclasses import dataclass
from types import ModuleType
from typing import Callable, Generic, Optional, TypeVar, cast

import msgspec
import pytest
import yaml
from typing_extensions import Annotated

from litestar import Controller, Litestar, Router, delete, get, patch, post
from litestar._openapi.plugin import OpenAPIPlugin
from litestar.enums import MediaType, OpenAPIMediaType, ParamType
from litestar.openapi import OpenAPIConfig, OpenAPIController
from litestar.openapi.spec import Parameter as OpenAPIParameter
from litestar.params import Parameter
from litestar.serialization.msgspec_hooks import decode_json, encode_json, get_serializer
from litestar.status_codes import HTTP_200_OK, HTTP_404_NOT_FOUND
from litestar.testing import create_test_client

CREATE_EXAMPLES_VALUES = (True, False)


@pytest.fixture(params=[True, False])
def create_examples(request: pytest.FixtureRequest) -> bool:
    return request.param  # type: ignore[no-any-return]


@pytest.mark.parametrize("schema_path", ["/schema/openapi.yaml", "/schema/openapi.yml"])
def test_openapi(
    person_controller: type[Controller],
    pet_controller: type[Controller],
    create_examples: bool,
    schema_path: str,
    openapi_controller: type[OpenAPIController] | None,
) -> None:
    openapi_config = OpenAPIConfig(
        "Example API", "1.0.0", create_examples=create_examples, openapi_controller=openapi_controller
    )
    with create_test_client([person_controller, pet_controller], openapi_config=openapi_config) as client:
        assert client.app.openapi_schema
        openapi_schema = client.app.openapi_schema
        assert openapi_schema.paths
        response = client.get(schema_path)
        assert response.status_code == HTTP_200_OK
        assert response.headers["content-type"] == OpenAPIMediaType.OPENAPI_YAML.value
        assert client.app.openapi_schema
        serializer = get_serializer(client.app.type_encoders)
        schema_json = decode_json(encode_json(openapi_schema.to_schema(), serializer))
        assert response.content.decode("utf-8") == yaml.dump(schema_json)


def test_openapi_json(
    person_controller: type[Controller],
    pet_controller: type[Controller],
    create_examples: bool,
    openapi_controller: type[OpenAPIController] | None,
) -> None:
    openapi_config = OpenAPIConfig(
        "Example API", "1.0.0", create_examples=create_examples, openapi_controller=openapi_controller
    )
    with create_test_client([person_controller, pet_controller], openapi_config=openapi_config) as client:
        assert client.app.openapi_schema
        openapi_schema = client.app.openapi_schema
        assert openapi_schema.paths
        response = client.get("/schema/openapi.json")
        assert response.status_code == HTTP_200_OK
        assert response.headers["content-type"] == OpenAPIMediaType.OPENAPI_JSON.value
        assert client.app.openapi_schema
        serializer = get_serializer(client.app.type_encoders)
        assert response.content == encode_json(openapi_schema.to_schema(), serializer)


@pytest.mark.parametrize(
    "endpoint, schema_path", [("openapi.yaml", "/schema/openapi.yaml"), ("openapi.yml", "/schema/openapi.yml")]
)
def test_openapi_yaml_not_allowed(
    endpoint: str,
    schema_path: str,
    person_controller: type[Controller],
    pet_controller: type[Controller],
    openapi_controller: type[OpenAPIController] | None,
) -> None:
    openapi_config = OpenAPIConfig(
        "Example API", "1.0.0", enabled_endpoints=set(), openapi_controller=openapi_controller
    )

    with create_test_client([person_controller, pet_controller], openapi_config=openapi_config) as client:
        assert client.app.openapi_schema
        openapi_schema = client.app.openapi_schema
        assert openapi_schema.paths
        response = client.get(schema_path)
        assert response.status_code == HTTP_404_NOT_FOUND


def test_openapi_json_not_allowed(person_controller: type[Controller], pet_controller: type[Controller]) -> None:
    # only tested with the OpenAPIController, b/c new router based approach always serves `openapi.json`.
    openapi_config = OpenAPIConfig(
        "Example API",
        "1.0.0",
        enabled_endpoints=set(),
        openapi_controller=OpenAPIController,
    )

    with create_test_client([person_controller, pet_controller], openapi_config=openapi_config) as client:
        assert client.app.openapi_schema
        openapi_schema = client.app.openapi_schema
        assert openapi_schema.paths
        response = client.get("/schema/openapi.json")
        assert response.status_code == HTTP_404_NOT_FOUND


def test_openapi_custom_path(openapi_controller: type[OpenAPIController] | None) -> None:
    openapi_config = OpenAPIConfig(
        title="my title", version="1.0.0", path="/custom_schema_path", openapi_controller=openapi_controller
    )
    with create_test_client([], openapi_config=openapi_config) as client:
        response = client.get("/schema")
        assert response.status_code == HTTP_404_NOT_FOUND

        response = client.get("/custom_schema_path")
        assert response.status_code == HTTP_200_OK

        response = client.get("/custom_schema_path/openapi.json")
        assert response.status_code == HTTP_200_OK


def test_openapi_normalizes_custom_path(openapi_controller: type[OpenAPIController] | None) -> None:
    openapi_config = OpenAPIConfig(
        title="my title", version="1.0.0", path="custom_schema_path", openapi_controller=openapi_controller
    )
    with create_test_client([], openapi_config=openapi_config) as client:
        response = client.get("/custom_schema_path/openapi.json")
        assert response.status_code == HTTP_200_OK

        response = client.get("/custom_schema_path/openapi.json")
        assert response.status_code == HTTP_200_OK


def test_openapi_custom_path_avoids_override() -> None:
    class CustomOpenAPIController(OpenAPIController):
        path = "/custom_docs"

    openapi_config = OpenAPIConfig(title="my title", version="1.0.0", openapi_controller=CustomOpenAPIController)
    with create_test_client([], openapi_config=openapi_config) as client:
        response = client.get("/schema")
        assert response.status_code == HTTP_404_NOT_FOUND

        response = client.get("/custom_docs/openapi.json")
        assert response.status_code == HTTP_200_OK

        response = client.get("/custom_docs/openapi.json")
        assert response.status_code == HTTP_200_OK


def test_openapi_custom_path_overrides_custom_controller_path() -> None:
    class CustomOpenAPIController(OpenAPIController):
        path = "/custom_docs"

    openapi_config = OpenAPIConfig(
        title="my title", version="1.0.0", openapi_controller=CustomOpenAPIController, path="/override_docs_path"
    )
    with create_test_client([], openapi_config=openapi_config) as client:
        response = client.get("/custom_docs")
        assert response.status_code == HTTP_404_NOT_FOUND

        response = client.get("/override_docs_path/openapi.json")
        assert response.status_code == HTTP_200_OK

        response = client.get("/override_docs_path/openapi.json")
        assert response.status_code == HTTP_200_OK


def test_msgspec_schema_generation(create_examples: bool, openapi_controller: type[OpenAPIController] | None) -> None:
    class Lookup(msgspec.Struct):
        id: Annotated[
            str,
            msgspec.Meta(
                min_length=12,
                max_length=16,
                description="A unique identifier",
                examples=["e4eaaaf2-d142-11e1-b3e4-080027620cdd"],
            ),
        ]

    @post("/example")
    async def example_route() -> Lookup:
        return Lookup(id="1234567812345678")

    with create_test_client(
        route_handlers=[example_route],
        openapi_config=OpenAPIConfig(
            title="Example API",
            version="1.0.0",
            create_examples=create_examples,
            openapi_controller=openapi_controller,
        ),
        signature_types=[Lookup],
    ) as client:
        response = client.get("/schema/openapi.json")
        assert response.status_code == HTTP_200_OK
        assert response.json()["components"]["schemas"]["test_msgspec_schema_generation.Lookup"]["properties"][
            "id"
        ] == {
            "description": "A unique identifier",
            "examples": ["e4eaaaf2-d142-11e1-b3e4-080027620cdd"],
            "maxLength": 16,
            "minLength": 12,
            "type": "string",
        }


def test_dataclass_field_default() -> None:
    # https://github.com/litestar-org/litestar/issues/3201
    @dataclass
    class SomeModel:
        field_a: str = "default_a"
        field_b: str = dataclasses.field(default="default_b")
        field_c: str = dataclasses.field(default_factory=lambda: "default_c")

    @get("/")
    async def handler() -> SomeModel:
        return SomeModel()

    app = Litestar(route_handlers=[handler], signature_types=[SomeModel])
    schema = app.openapi_schema.components.schemas["test_dataclass_field_default.SomeModel"]
    assert schema
    assert schema.properties["field_a"].default == "default_a"  # type: ignore[union-attr, index]
    assert schema.properties["field_b"].default == "default_b"  # type: ignore[union-attr, index]
    assert schema.properties["field_c"].default is None  # type: ignore[union-attr, index]


def test_struct_field_default() -> None:
    # https://github.com/litestar-org/litestar/issues/3201
    class SomeModel(msgspec.Struct, kw_only=True):
        field_a: str = "default_a"
        field_b: str = msgspec.field(default="default_b")
        field_c: str = msgspec.field(default_factory=lambda: "default_c")

    @get("/")
    async def handler() -> SomeModel:
        return SomeModel()

    app = Litestar(route_handlers=[handler], signature_types=[SomeModel])
    schema = app.openapi_schema.components.schemas["test_struct_field_default.SomeModel"]
    assert schema
    assert schema.properties["field_a"].default == "default_a"  # type: ignore[union-attr, index]
    assert schema.properties["field_b"].default == "default_b"  # type: ignore[union-attr, index]
    assert schema.properties["field_c"].default is None  # type: ignore[union-attr, index]


def test_schema_for_optional_path_parameter(openapi_controller: type[OpenAPIController] | None) -> None:
    @get(path=["/", "/{test_message:str}"], media_type=MediaType.TEXT, sync_to_thread=False)
    def handler(test_message: Optional[str]) -> str:  # noqa: UP007
        return test_message or "no message"

    with create_test_client(
        route_handlers=[handler],
        openapi_config=OpenAPIConfig(
            title="Example API",
            version="1.0.0",
            create_examples=True,
            openapi_controller=openapi_controller,
        ),
    ) as client:
        response = client.get("/schema/openapi.json")
        assert response.status_code == HTTP_200_OK
        assert "parameters" not in response.json()["paths"]["/"]["get"]  # type[ignore]
        parameter = response.json()["paths"]["/{test_message}"]["get"]["parameters"][0]  # type[ignore]
        assert parameter
        assert parameter["in"] == ParamType.PATH
        assert parameter["name"] == "test_message"


T = TypeVar("T")


@dataclass
class Foo(Generic[T]):
    foo: T


def test_with_generic_class(openapi_controller: type[OpenAPIController] | None) -> None:
    @get("/foo-str", sync_to_thread=False)
    def handler_foo_str() -> Foo[str]:
        return Foo("")

    @get("/foo-int", sync_to_thread=False)
    def handler_foo_int() -> Foo[int]:
        return Foo(1)

    with create_test_client(
        route_handlers=[handler_foo_str, handler_foo_int],
        openapi_config=OpenAPIConfig(
            title="Example API",
            version="1.0.0",
            openapi_controller=openapi_controller,
        ),
    ) as client:
        response = client.get("/schema/openapi.json")
        assert response.status_code == HTTP_200_OK
        assert response.json() == {
            "info": {"title": "Example API", "version": "1.0.0"},
            "openapi": "3.1.0",
            "servers": [{"url": "/"}],
            "paths": {
                "/foo-str": {
                    "get": {
                        "summary": "HandlerFooStr",
                        "operationId": "FooStrHandlerFooStr",
                        "responses": {
                            "200": {
                                "description": "Request fulfilled, document follows",
                                "headers": {},
                                "content": {"application/json": {"schema": {"$ref": "#/components/schemas/Foo[str]"}}},
                            }
                        },
                        "deprecated": False,
                    }
                },
                "/foo-int": {
                    "get": {
                        "summary": "HandlerFooInt",
                        "operationId": "FooIntHandlerFooInt",
                        "responses": {
                            "200": {
                                "description": "Request fulfilled, document follows",
                                "headers": {},
                                "content": {"application/json": {"schema": {"$ref": "#/components/schemas/Foo[int]"}}},
                            }
                        },
                        "deprecated": False,
                    }
                },
            },
            "components": {
                "schemas": {
                    "Foo[str]": {
                        "properties": {"foo": {"type": "string"}},
                        "type": "object",
                        "required": ["foo"],
                        "title": "Foo[str]",
                    },
                    "Foo[int]": {
                        "properties": {"foo": {"type": "integer"}},
                        "type": "object",
                        "required": ["foo"],
                        "title": "Foo[int]",
                    },
                }
            },
        }


def test_allow_multiple_parameters_with_same_name_but_different_location() -> None:
    """Test that we can support params with the same name if they are in different locations, e.g., cookie and header.

    https://github.com/litestar-org/litestar/issues/2662
    """

    @post("/test")
    async def route(
        name: Annotated[Optional[str], Parameter(cookie="name")] = None,  # noqa: UP007
        name_header: Annotated[Optional[str], Parameter(header="name")] = None,  # noqa: UP007
    ) -> str:
        return name or name_header or ""

    app = Litestar(route_handlers=[route], debug=True)
    assert app.openapi_schema.paths is not None
    schema = app.openapi_schema
    paths = schema.paths
    assert paths is not None
    path = paths["/test"]
    assert path.post is not None
    parameters = path.post.parameters
    assert parameters is not None
    assert len(parameters) == 2
    assert all(isinstance(param, OpenAPIParameter) for param in parameters)
    params = cast("list[OpenAPIParameter]", parameters)
    assert all(param.name == "name" for param in params)
    assert tuple(param.param_in for param in params) == ("cookie", "header")


def test_schema_name_collisions(create_module: Callable[[str], ModuleType]) -> None:
    module_a = create_module(
        """
from dataclasses import dataclass

@dataclass
class Model:
    a: str

"""
    )

    module_b = create_module(
        """
from dataclasses import dataclass

@dataclass
class Model:
    b: str

"""
    )

    @get("/foo", sync_to_thread=False, signature_namespace={"module_a": module_a})
    def handler_a() -> module_a.Model:  # type: ignore[name-defined]
        return module_a.Model(a="")

    @get("/bar", sync_to_thread=False, signature_namespace={"module_b": module_b})
    def handler_b() -> module_b.Model:  # type: ignore[name-defined]
        return module_b.Model(b="")

    app = Litestar(route_handlers=[handler_a, handler_b], debug=True)
    openapi_plugin = app.plugins.get(OpenAPIPlugin)
    assert openapi_plugin.provide_openapi().components.schemas.keys() == {
        f"{module_a.__name__}_Model",
        f"{module_b.__name__}_Model",
    }
    # TODO: expand this test to cover more cases


def test_multiple_handlers_for_same_route() -> None:
    @post("/", sync_to_thread=False)
    def post_handler() -> None: ...

    @get("/", sync_to_thread=False)
    def get_handler() -> None: ...

    app = Litestar([get_handler, post_handler])
    openapi_plugin = app.plugins.get(OpenAPIPlugin)
    openapi = openapi_plugin.provide_openapi()

    assert openapi.paths is not None
    path_item = openapi.paths["/"]
    assert path_item.get is not None
    assert path_item.post is not None


@pytest.mark.parametrize(("random_seed_one", "random_seed_two", "should_be_equal"), [(10, 10, True), (10, 20, False)])
def test_seeding(random_seed_one: int, random_seed_two: int, should_be_equal: bool) -> None:
    @post("/", sync_to_thread=False)
    def post_handler(q: str) -> None: ...

    @get("/", sync_to_thread=False)
    def get_handler(q: str) -> None: ...

    app = Litestar(
        [get_handler, post_handler], openapi_config=OpenAPIConfig("Litestar", "v0.0.1", True, random_seed_one)
    )
    openapi_plugin = app.plugins.get(OpenAPIPlugin)
    openapi_one = openapi_plugin.provide_openapi()

    app = Litestar(
        [get_handler, post_handler], openapi_config=OpenAPIConfig("Litestar", "v0.0.1", True, random_seed_two)
    )
    openapi_plugin = app.plugins.get(OpenAPIPlugin)
    openapi_two = openapi_plugin.provide_openapi()

    if should_be_equal:
        assert openapi_one == openapi_two
    else:
        assert openapi_one != openapi_two


def test_components_schemas_in_alphabetical_order() -> None:
    # https://github.com/litestar-org/litestar/issues/3059

    @dataclass
    class A: ...

    @dataclass
    class B: ...

    @dataclass
    class C: ...

    class TestController(Controller):
        @post("/", sync_to_thread=False)
        def post_handler(self, data: B) -> None: ...

        @get("/", sync_to_thread=False)
        def get_handler(self) -> A:  # type: ignore[empty-body]
            ...

        @patch("/", sync_to_thread=False)
        def patch_handler(self, data: C) -> A:  # type: ignore[empty-body]
            ...

        @delete("/", sync_to_thread=False)
        def delete_handler(self, data: B) -> None: ...

    app = Litestar([TestController], signature_types=[A, B, C])
    openapi_plugin = app.plugins.get(OpenAPIPlugin)
    openapi = openapi_plugin.provide_openapi()

    expected_keys = [
        "test_components_schemas_in_alphabetical_order.A",
        "test_components_schemas_in_alphabetical_order.B",
        "test_components_schemas_in_alphabetical_order.C",
    ]
    assert list(openapi.components.schemas.keys()) == expected_keys


def test_openapi_controller_and_openapi_router_on_same_app() -> None:
    """Test that OpenAPIController and OpenAPIRouter can coexist on the same app.

    As part of backward compatibility with new plugin-based OpenAPI router approach, we did not consider
    the case where an OpenAPIController is registered on the application by means other than via the
    OpenAPIConfig object. This is an approach that has been used to serve the openapi both under the
    `/schema` and `/some-prefix/schema` paths. This test ensures that the OpenAPIController and OpenAPIRouter
    can coexist on the same app.

    See: https://github.com/litestar-org/litestar/issues/3337
    """
    router = Router(path="/abc", route_handlers=[OpenAPIController])
    openapi_config = OpenAPIConfig("Litestar", "v0.0.1")  # no openapi_controller specified means we use the router
    app = Litestar([router], openapi_config=openapi_config)
    assert sorted(r.path for r in app.routes) == [
        "/abc/schema",
        "/abc/schema/elements",
        "/abc/schema/oauth2-redirect.html",
        "/abc/schema/openapi.json",
        "/abc/schema/openapi.yaml",
        "/abc/schema/openapi.yml",
        "/abc/schema/rapidoc",
        "/abc/schema/redoc",
        "/abc/schema/swagger",
        "/schema",
        "/schema/elements",
        "/schema/oauth2-redirect.html",
        "/schema/openapi.json",
        "/schema/openapi.yaml",
        "/schema/openapi.yml",
        "/schema/rapidoc",
        "/schema/redoc",
        "/schema/swagger",
        "/schema/{path:str}",
    ]



from uuid import uuid4

from docs.examples.security.jwt.using_jwt_auth import app

from litestar.status_codes import HTTP_200_OK, HTTP_201_CREATED, HTTP_401_UNAUTHORIZED
from litestar.testing import TestClient


def test_using_jwt_auth() -> None:
    with TestClient(app) as client:
        response = client.get("/some-path")
        assert response.status_code == HTTP_401_UNAUTHORIZED
        response = client.post(
            "/login", json={"name": "Moishe Zuchmir", "email": "moishe@zuchmir.com", "id": str(uuid4())}
        )
        assert response.status_code == HTTP_201_CREATED, response.json()
        response = client.get("/some-path", headers={"Authorization": response.headers.get("authorization")})
        assert response.status_code == HTTP_200_OK



from __future__ import annotations

import sys
from abc import ABC, abstractmethod
from dataclasses import asdict, dataclass, field, fields
from importlib.util import find_spec
from logging import INFO
from typing import TYPE_CHECKING, Any, Callable, Literal, cast

from litestar.exceptions import ImproperlyConfiguredException, MissingDependencyException
from litestar.serialization.msgspec_hooks import _msgspec_json_encoder
from litestar.utils.deprecation import deprecated

__all__ = ("BaseLoggingConfig", "LoggingConfig", "StructLoggingConfig")


if TYPE_CHECKING:
    from collections.abc import Iterable
    from typing import NoReturn

    # these imports are duplicated on purpose so sphinx autodoc can find and link them
    from structlog.types import BindableLogger, Processor, WrappedLogger
    from structlog.typing import EventDict

    from litestar.types import Logger, Scope
    from litestar.types.callable_types import ExceptionLoggingHandler, GetLogger


default_handlers: dict[str, dict[str, Any]] = {
    "console": {
        "class": "logging.StreamHandler",
        "level": "DEBUG",
        "formatter": "standard",
    },
    "queue_listener": {
        "class": "litestar.logging.standard.QueueListenerHandler",
        "level": "DEBUG",
        "formatter": "standard",
    },
}

if sys.version_info >= (3, 12, 0):
    default_handlers["queue_listener"].update(
        {
            "class": "logging.handlers.QueueHandler",
            "queue": {
                "()": "queue.Queue",
                "maxsize": -1,
            },
            "listener": "litestar.logging.standard.LoggingQueueListener",
            "handlers": ["console"],
        }
    )

    # do not format twice, the console handler will do the job
    del default_handlers["queue_listener"]["formatter"]


default_picologging_handlers: dict[str, dict[str, Any]] = {
    "console": {
        "class": "picologging.StreamHandler",
        "level": "DEBUG",
        "formatter": "standard",
    },
    "queue_listener": {
        "class": "litestar.logging.picologging.QueueListenerHandler",
        "level": "DEBUG",
        "formatter": "standard",
    },
}


def get_logger_placeholder(_: str | None = None) -> NoReturn:
    """Raise: An :class:`ImproperlyConfiguredException <.exceptions.ImproperlyConfiguredException>`"""
    raise ImproperlyConfiguredException(
        "cannot call '.get_logger' without passing 'logging_config' to the Litestar constructor first"
    )


def _get_default_handlers() -> dict[str, dict[str, Any]]:
    """Return the default logging handlers for the config.

    Returns:
        A dictionary of logging handlers
    """
    if find_spec("picologging"):
        return default_picologging_handlers
    return default_handlers


def _default_exception_logging_handler_factory(
    is_struct_logger: bool, traceback_line_limit: int
) -> ExceptionLoggingHandler:
    """Create an exception logging handler function.

    Args:
        is_struct_logger: Whether the logger is a structlog instance.
        traceback_line_limit: Maximal number of lines to log from the
            traceback.

    Returns:
        An exception logging handler.
    """

    def _default_exception_logging_handler(logger: Logger, scope: Scope, tb: list[str]) -> None:
        # we limit the length of the stack trace to 20 lines.
        first_line = tb.pop(0)

        if is_struct_logger:
            logger.exception(
                "Uncaught Exception",
                connection_type=scope["type"],
                path=scope["path"],
                traceback="".join(tb[-traceback_line_limit:]),
            )
        else:
            stack_trace = first_line + "".join(tb[-traceback_line_limit:])
            logger.exception(
                "exception raised on %s connection to route %s\n\n%s", scope["type"], scope["path"], stack_trace
            )

    return _default_exception_logging_handler


class BaseLoggingConfig(ABC):
    """Abstract class that should be extended by logging configs."""

    __slots__ = ("log_exceptions", "traceback_line_limit", "exception_logging_handler")

    log_exceptions: Literal["always", "debug", "never"]
    """Should exceptions be logged, defaults to log exceptions when ``app.debug == True``'"""
    traceback_line_limit: int
    """Max number of lines to print for exception traceback"""
    exception_logging_handler: ExceptionLoggingHandler | None
    """Handler function for logging exceptions."""

    @abstractmethod
    def configure(self) -> GetLogger:
        """Return logger with the given configuration.

        Returns:
            A 'logging.getLogger' like function.
        """
        raise NotImplementedError("abstract method")

    @staticmethod
    def set_level(logger: Any, level: int) -> None:
        """Provides a consistent interface to call `setLevel` for all loggers."""
        raise NotImplementedError("abstract method")


@dataclass
class LoggingConfig(BaseLoggingConfig):
    """Configuration class for standard logging.

    Notes:
        - If 'picologging' is installed it will be used by default.
    """

    version: Literal[1] = field(default=1)
    """The only valid value at present is 1."""
    incremental: bool = field(default=False)
    """Whether the configuration is to be interpreted as incremental to the existing configuration.

    Notes:
        - This option is ignored for 'picologging'
    """
    disable_existing_loggers: bool = field(default=False)
    """Whether any existing non-root loggers are to be disabled."""
    filters: dict[str, dict[str, Any]] | None = field(default=None)
    """A dict in which each key is a filter id and each value is a dict describing how to configure the corresponding
    Filter instance.
    """
    propagate: bool = field(default=True)
    """If messages must propagate to handlers higher up the logger hierarchy from this logger."""
    formatters: dict[str, dict[str, Any]] = field(
        default_factory=lambda: {
            "standard": {"format": "%(levelname)s - %(asctime)s - %(name)s - %(module)s - %(message)s"}
        }
    )
    handlers: dict[str, dict[str, Any]] = field(default_factory=_get_default_handlers)
    """A dict in which each key is a handler id and each value is a dict describing how to configure the corresponding
    Handler instance.
    """
    loggers: dict[str, dict[str, Any]] = field(
        default_factory=lambda: {
            "litestar": {"level": "INFO", "handlers": ["queue_listener"], "propagate": False},
        }
    )
    """A dict in which each key is a logger name and each value is a dict describing how to configure the corresponding
    Logger instance.
    """
    root: dict[str, dict[str, Any] | list[Any] | str] = field(
        default_factory=lambda: {
            "handlers": ["queue_listener"],
            "level": "INFO",
        }
    )
    """This will be the configuration for the root logger.

    Processing of the configuration will be as for any logger, except that the propagate setting will not be applicable.
    """
    configure_root_logger: bool = field(default=True)
    """Should the root logger be configured, defaults to True for ease of configuration."""
    log_exceptions: Literal["always", "debug", "never"] = field(default="debug")
    """Should exceptions be logged, defaults to log exceptions when 'app.debug == True'"""
    traceback_line_limit: int = field(default=20)
    """Max number of lines to print for exception traceback"""
    exception_logging_handler: ExceptionLoggingHandler | None = field(default=None)
    """Handler function for logging exceptions."""

    def __post_init__(self) -> None:
        if "queue_listener" not in self.handlers:
            self.handlers["queue_listener"] = _get_default_handlers()["queue_listener"]

        if "litestar" not in self.loggers:
            self.loggers["litestar"] = {
                "level": "INFO",
                "handlers": ["queue_listener"],
                "propagate": False,
            }

        if self.log_exceptions != "never" and self.exception_logging_handler is None:
            self.exception_logging_handler = _default_exception_logging_handler_factory(
                is_struct_logger=False, traceback_line_limit=self.traceback_line_limit
            )

    def configure(self) -> GetLogger:
        """Return logger with the given configuration.

        Returns:
            A 'logging.getLogger' like function.
        """

        excluded_fields: tuple[str, ...]
        if "picologging" in " ".join([handler["class"] for handler in self.handlers.values()]):
            try:
                from picologging import config, getLogger
            except ImportError as e:
                raise MissingDependencyException("picologging") from e

            excluded_fields = ("incremental", "configure_root_logger")
        else:
            from logging import config, getLogger  # type: ignore[no-redef, assignment]

            excluded_fields = ("configure_root_logger",)

        values = {
            _field.name: getattr(self, _field.name)
            for _field in fields(self)
            if getattr(self, _field.name) is not None and _field.name not in excluded_fields
        }

        if not self.configure_root_logger:
            values.pop("root")
        config.dictConfig(values)
        return cast("Callable[[str], Logger]", getLogger)

    @staticmethod
    def set_level(logger: Logger, level: int) -> None:
        """Provides a consistent interface to call `setLevel` for all loggers."""
        logger.setLevel(level)


class StructlogEventFilter:
    """Remove keys from the log event.

    Add an instance to the processor chain.

    .. code-block:: python
        :caption: Examples

        structlog.configure(
            ...,
            processors=[
                ...,
                EventFilter(["color_message"]),
                ...,
            ],
        )

    """

    def __init__(self, filter_keys: Iterable[str]) -> None:
        """Initialize the EventFilter.

        Args:
            filter_keys: Iterable of string keys to be excluded from the log event.
        """
        self.filter_keys = filter_keys

    def __call__(self, _: WrappedLogger, __: str, event_dict: EventDict) -> EventDict:
        """Receive the log event, and filter keys.

        Args:
            _ ():
            __ ():
            event_dict (): The data to be logged.

        Returns:
            The log event with any key in `self.filter_keys` removed.
        """
        for key in self.filter_keys:
            event_dict.pop(key, None)
        return event_dict


def default_json_serializer(value: EventDict, **_: Any) -> bytes:
    return _msgspec_json_encoder.encode(value)


def stdlib_json_serializer(value: EventDict, **_: Any) -> str:  # pragma: no cover
    return _msgspec_json_encoder.encode(value).decode("utf-8")


def default_structlog_processors(as_json: bool = True) -> list[Processor]:  # pyright: ignore
    """Set the default processors for structlog.

    Returns:
        An optional list of processors.
    """
    try:
        import structlog
        from structlog.dev import RichTracebackFormatter

        if as_json:
            return [
                structlog.contextvars.merge_contextvars,
                structlog.processors.add_log_level,
                structlog.processors.format_exc_info,
                structlog.processors.TimeStamper(fmt="iso"),
                structlog.processors.JSONRenderer(serializer=default_json_serializer),
            ]
        return [
            structlog.contextvars.merge_contextvars,
            structlog.processors.add_log_level,
            structlog.processors.TimeStamper(fmt="iso"),
            structlog.dev.ConsoleRenderer(
                colors=True, exception_formatter=RichTracebackFormatter(max_frames=1, show_locals=False, width=80)
            ),
        ]

    except ImportError:
        return []


def default_structlog_standard_lib_processors(as_json: bool = True) -> list[Processor]:  # pyright: ignore
    """Set the default processors for structlog stdlib.

    Returns:
        An optional list of processors.
    """
    try:
        import structlog
        from structlog.dev import RichTracebackFormatter

        if as_json:
            return [
                structlog.processors.TimeStamper(fmt="iso"),
                structlog.stdlib.add_log_level,
                structlog.stdlib.ExtraAdder(),
                StructlogEventFilter(["color_message"]),
                structlog.stdlib.ProcessorFormatter.remove_processors_meta,
                structlog.processors.JSONRenderer(serializer=stdlib_json_serializer),
            ]
        return [
            structlog.processors.TimeStamper(fmt="iso"),
            structlog.stdlib.add_log_level,
            structlog.stdlib.ExtraAdder(),
            StructlogEventFilter(["color_message"]),
            structlog.stdlib.ProcessorFormatter.remove_processors_meta,
            structlog.dev.ConsoleRenderer(
                colors=True, exception_formatter=RichTracebackFormatter(max_frames=1, show_locals=False, width=80)
            ),
        ]
    except ImportError:
        return []


def default_logger_factory(as_json: bool = True) -> Callable[..., WrappedLogger] | None:
    """Set the default logger factory for structlog.

    Returns:
        An optional logger factory.
    """
    try:
        import structlog

        if as_json:
            return structlog.BytesLoggerFactory()
        return structlog.WriteLoggerFactory()
    except ImportError:
        return None


@dataclass
class StructLoggingConfig(BaseLoggingConfig):
    """Configuration class for structlog.

    Notes:
        - requires ``structlog`` to be installed.
    """

    processors: list[Processor] | None = field(default=None)  # pyright: ignore
    """Iterable of structlog logging processors."""
    standard_lib_logging_config: LoggingConfig | None = field(default=None)  # pyright: ignore
    """Optional customized standard logging configuration.

    Use this when you need to modify the standard library outside of the Structlog pre-configured implementation.
    """
    wrapper_class: type[BindableLogger] | None = field(default=None)  # pyright: ignore
    """Structlog bindable logger."""
    context_class: dict[str, Any] | None = None
    """Context class (a 'contextvar' context) for the logger."""
    logger_factory: Callable[..., WrappedLogger] | None = field(default=None)  # pyright: ignore
    """Logger factory to use."""
    cache_logger_on_first_use: bool = field(default=True)
    """Whether to cache the logger configuration and reuse."""
    log_exceptions: Literal["always", "debug", "never"] = field(default="debug")
    """Should exceptions be logged, defaults to log exceptions when 'app.debug == True'"""
    traceback_line_limit: int = field(default=20)
    """Max number of lines to print for exception traceback"""
    exception_logging_handler: ExceptionLoggingHandler | None = field(default=None)
    """Handler function for logging exceptions."""
    pretty_print_tty: bool = field(default=True)
    """Pretty print log output when run from an interactive terminal."""

    def __post_init__(self) -> None:
        if self.processors is None:
            self.processors = default_structlog_processors(not sys.stderr.isatty() and self.pretty_print_tty)
        if self.logger_factory is None:
            self.logger_factory = default_logger_factory(not sys.stderr.isatty() and self.pretty_print_tty)
        if self.log_exceptions != "never" and self.exception_logging_handler is None:
            self.exception_logging_handler = _default_exception_logging_handler_factory(
                is_struct_logger=True, traceback_line_limit=self.traceback_line_limit
            )
        try:
            import structlog

            if self.standard_lib_logging_config is None:
                self.standard_lib_logging_config = LoggingConfig(
                    formatters={
                        "standard": {
                            "()": structlog.stdlib.ProcessorFormatter,
                            "processors": default_structlog_standard_lib_processors(
                                as_json=not sys.stderr.isatty() and self.pretty_print_tty
                            ),
                        }
                    }
                )
        except ImportError:
            self.standard_lib_logging_config = LoggingConfig()

    def configure(self) -> GetLogger:
        """Return logger with the given configuration.

        Returns:
            A 'logging.getLogger' like function.
        """
        try:
            import structlog
        except ImportError as e:
            raise MissingDependencyException("structlog") from e

        structlog.configure(
            **{
                k: v
                for k, v in asdict(self).items()
                if k
                not in (
                    "standard_lib_logging_config",
                    "log_exceptions",
                    "traceback_line_limit",
                    "exception_logging_handler",
                    "pretty_print_tty",
                )
            }
        )
        return structlog.get_logger

    @staticmethod
    def set_level(logger: Logger, level: int) -> None:
        """Provides a consistent interface to call `setLevel` for all loggers."""

        try:
            import structlog

            structlog.configure(wrapper_class=structlog.make_filtering_bound_logger(level))
        except ImportError:
            """"""
            return


@deprecated(version="2.6.0", removal_in="3.0.0", alternative="`StructLoggingConfig.set_level`")
def default_wrapper_class(log_level: int = INFO) -> type[BindableLogger] | None:  # pragma: no cover  # pyright: ignore
    try:  # pragma: no cover
        import structlog

        return structlog.make_filtering_bound_logger(log_level)
    except ImportError:
        return None



import io
import os
import re
import sys
from pathlib import Path
from typing import Callable, Generator, List, Literal, Optional, Tuple, Union
from unittest.mock import MagicMock

import pytest
from _pytest.monkeypatch import MonkeyPatch
from click.testing import CliRunner
from pytest_mock import MockerFixture
from rich.console import Console

from litestar import __version__ as litestar_version
from litestar.cli import _utils
from litestar.cli.commands import core
from litestar.cli.main import litestar_group as cli_command
from litestar.exceptions import LitestarWarning

from . import (
    APP_FACTORY_FILE_CONTENT_SERVER_LIFESPAN_PLUGIN,
    APP_FILE_CONTENT_ROUTES_EXAMPLE,
    CREATE_APP_FILE_CONTENT,
    GENERIC_APP_FACTORY_FILE_CONTENT,
    GENERIC_APP_FACTORY_FILE_CONTENT_STRING_ANNOTATION,
)
from .conftest import CreateAppFileFixture

project_base = Path(__file__).parent.parent.parent


@pytest.fixture()
def mock_show_app_info(mocker: MockerFixture) -> MagicMock:
    return mocker.patch("litestar.cli.commands.core.show_app_info")


@pytest.mark.parametrize("set_in_env", [True, False])
@pytest.mark.parametrize(
    "host, port, uds, fd",
    [("0.0.0.0", 8081, "/run/uvicorn/litestar_test.sock", 0), (None, None, None, None)],
)
@pytest.mark.parametrize("custom_app_file,", [Path("my_app.py"), None])
@pytest.mark.parametrize("app_dir", ["custom_subfolder", None])
@pytest.mark.parametrize(
    "reload, reload_dir, reload_include, reload_exclude, web_concurrency",
    [
        (None, None, None, None, None),
        (True, None, None, None, None),
        (False, None, None, None, None),
        (True, [".", "../somewhere_else"], None, None, None),
        (False, [".", "../somewhere_else"], None, None, None),
        (True, None, ["*.rst", "*.yml"], None, None),
        (False, None, None, ["*.py"], None),
        (False, None, ["*.yml", "*.rst"], None, None),
        (None, None, None, None, 2),
        (True, None, None, None, 2),
        (False, None, None, None, 2),
    ],
)
@pytest.mark.parametrize("tty_enabled", [True, False])
@pytest.mark.parametrize("quiet_console", [True, False])
def test_run_command(
    mock_show_app_info: MagicMock,
    mocker: MockerFixture,
    runner: CliRunner,
    monkeypatch: MonkeyPatch,
    reload: Optional[bool],
    port: Optional[int],
    host: Optional[str],
    fd: Optional[int],
    uds: Optional[str],
    web_concurrency: Optional[int],
    app_dir: Optional[str],
    reload_dir: Optional[List[str]],
    reload_include: Optional[List[str]],
    reload_exclude: Optional[List[str]],
    custom_app_file: Optional[Path],
    create_app_file: CreateAppFileFixture,
    set_in_env: bool,
    tty_enabled: bool,
    quiet_console: bool,
    mock_subprocess_run: MagicMock,
    mock_uvicorn_run: MagicMock,
    tmp_project_dir: Path,
) -> None:
    monkeypatch.delenv("LITESTAR_QUIET_CONSOLE", raising=False)
    if quiet_console:
        monkeypatch.setenv("LITESTAR_QUIET_CONSOLE", "true")
    mocker.patch.object(core, "isatty", return_value=tty_enabled)
    mocker.patch.object(_utils, "isatty", return_value=tty_enabled)
    args = []
    if custom_app_file:
        args.extend(["--app", f"{custom_app_file.stem}:app"])
    if app_dir is not None:
        args.extend(["--app-dir", str(Path(tmp_project_dir / app_dir))])
    args.extend(["run"])

    if reload:
        if set_in_env:
            monkeypatch.setenv("LITESTAR_RELOAD", "true")
        else:
            args.append("--reload")
    else:
        reload = False

    if port:
        if set_in_env:
            monkeypatch.setenv("LITESTAR_PORT", str(port))
        else:
            args.extend(["--port", str(port)])
    else:
        port = 8000

    if host:
        if set_in_env:
            monkeypatch.setenv("LITESTAR_HOST", host)
        else:
            args.extend(["--host", host])
    else:
        host = "127.0.0.1"

    if uds:
        if set_in_env:
            monkeypatch.setenv("LITESTAR_UNIX_DOMAIN_SOCKET", uds)
        else:
            args.extend(["--uds", uds])
    else:
        uds = None

    if fd is not None:
        if set_in_env:
            monkeypatch.setenv("LITESTAR_FILE_DESCRIPTOR", str(fd))
        else:
            args.extend(["--fd", str(fd)])
    else:
        fd = None

    if web_concurrency is None:
        web_concurrency = 1
    elif set_in_env:
        monkeypatch.setenv("LITESTAR_WEB_CONCURRENCY", str(web_concurrency))
    else:
        args.extend(["--web-concurrency", str(web_concurrency)])

    if reload_dir is not None:
        if set_in_env:
            monkeypatch.setenv("LITESTAR_RELOAD_DIRS", ",".join(reload_dir))
        else:
            args.extend([f"--reload-dir={s}" for s in reload_dir])

    if reload_include is not None:
        if set_in_env:
            monkeypatch.setenv("LITESTAR_RELOAD_INCLUDES", ",".join(reload_include))
        else:
            args.extend([f"--reload-include={s}" for s in reload_include])

    if reload_exclude is not None:
        if set_in_env:
            monkeypatch.setenv("LITESTAR_RELOAD_EXCLUDES", ",".join(reload_exclude))
        else:
            args.extend([f"--reload-exclude={s}" for s in reload_exclude])

    path = create_app_file(custom_app_file or "app.py", directory=app_dir)
    result = runner.invoke(cli_command, args)

    assert result.exception is None
    assert result.exit_code == 0

    if reload or reload_dir or reload_include or reload_exclude or web_concurrency > 1:
        expected_args = [
            sys.executable,
            "-m",
            "uvicorn",
            f"{path.stem}:app",
            f"--host={host}",
            f"--port={port}",
        ]
        if fd is not None:
            expected_args.append(f"--fd={fd}")
        if uds is not None:
            expected_args.append(f"--uds={uds}")
        if reload or reload_dir or reload_include or reload_exclude:
            expected_args.append("--reload")
        if web_concurrency:
            expected_args.append(f"--workers={web_concurrency}")
        if reload_dir:
            expected_args.extend([f"--reload-dir={s}" for s in reload_dir])
        if reload_include:
            expected_args.extend([f"--reload-include={s}" for s in reload_include])
        if reload_exclude:
            expected_args.extend([f"--reload-exclude={s}" for s in reload_exclude])
        mock_subprocess_run.assert_called_once()
        assert sorted(mock_subprocess_run.call_args_list[0].args[0]) == sorted(expected_args)
    else:
        mock_subprocess_run.assert_not_called()
        mock_uvicorn_run.assert_called_once_with(
            app=f"{path.stem}:app",
            host=host,
            port=port,
            uds=uds,
            fd=fd,
            factory=False,
            ssl_certfile=None,
            ssl_keyfile=None,
        )

    if tty_enabled and not quiet_console:
        mock_show_app_info.assert_called_once()
    else:
        mock_show_app_info.assert_not_called()


@pytest.mark.parametrize("quiet_console", [True, False])
@pytest.mark.parametrize("tty_enabled", [True, False])
@pytest.mark.parametrize(
    "file_name,file_content,factory_name",
    [
        ("_create_app.py", CREATE_APP_FILE_CONTENT, "create_app"),
        ("_generic_app_factory.py", GENERIC_APP_FACTORY_FILE_CONTENT, "any_name"),
        ("_generic_app_factory_string_ann.py", GENERIC_APP_FACTORY_FILE_CONTENT_STRING_ANNOTATION, "any_name"),
    ],
    ids=["create-app", "generic", "generic-string-annotated"],
)
def test_run_command_with_autodiscover_app_factory(
    runner: CliRunner,
    mock_uvicorn_run: MagicMock,
    file_name: str,
    file_content: str,
    factory_name: str,
    patch_autodiscovery_paths: Callable[[List[str]], None],
    tty_enabled: bool,
    quiet_console: bool,
    create_app_file: CreateAppFileFixture,
    mocker: MockerFixture,
    monkeypatch: MonkeyPatch,
) -> None:
    monkeypatch.delenv("LITESTAR_QUIET_CONSOLE", raising=False)
    if quiet_console:
        monkeypatch.setenv("LITESTAR_QUIET_CONSOLE", "true")
    mocker.patch.object(core, "isatty", return_value=tty_enabled)
    mocker.patch.object(_utils, "isatty", return_value=tty_enabled)
    patch_autodiscovery_paths([file_name])
    path = create_app_file(file_name, content=file_content)
    result = runner.invoke(cli_command, "run")
    assert result.exception is None
    assert result.exit_code == 0

    mock_uvicorn_run.assert_called_once_with(
        app=f"{path.stem}:{factory_name}",
        host="127.0.0.1",
        port=8000,
        factory=True,
        uds=None,
        fd=None,
        ssl_certfile=None,
        ssl_keyfile=None,
    )
    if tty_enabled and not quiet_console:
        assert len(result.output) > 0
    else:
        assert len(result.output) == 0


@pytest.mark.parametrize("quiet_console", [True, False])
@pytest.mark.parametrize("tty_enabled", [True, False])
def test_run_command_with_app_factory(
    runner: CliRunner,
    mock_uvicorn_run: MagicMock,
    create_app_file: CreateAppFileFixture,
    tty_enabled: bool,
    quiet_console: bool,
    mocker: MockerFixture,
    monkeypatch: MonkeyPatch,
) -> None:
    monkeypatch.delenv("LITESTAR_QUIET_CONSOLE", raising=False)
    if quiet_console:
        monkeypatch.setenv("LITESTAR_QUIET_CONSOLE", "true")
    mocker.patch.object(core, "isatty", return_value=tty_enabled)
    mocker.patch.object(_utils, "isatty", return_value=tty_enabled)
    path = create_app_file("_create_app_with_path.py", content=CREATE_APP_FILE_CONTENT)
    app_path = f"{path.stem}:create_app"
    result = runner.invoke(cli_command, ["--app", app_path, "run"])

    assert result.exception is None
    assert result.exit_code == 0

    mock_uvicorn_run.assert_called_once_with(
        app=str(app_path),
        host="127.0.0.1",
        port=8000,
        factory=True,
        uds=None,
        fd=None,
        ssl_certfile=None,
        ssl_keyfile=None,
    )
    if tty_enabled and not quiet_console:
        assert len(result.output) > 0
    else:
        assert len(result.output) == 0


@pytest.mark.parametrize(
    "cli, env, expected",
    (
        (
            ("--reload", True),
            ("LITESTAR_RELOAD", False),
            "--reload",
        ),
        (
            ("--reload-dir", [".", "../somewhere_else"]),
            ("LITESTAR_RELOAD_DIRS", ["../somewhere_else3", "../somewhere_else2"]),
            ["--reload-dir=.", "--reload-dir=../somewhere_else"],
        ),
        (
            ("--reload-include", ["*.rst", "*.yml"]),
            ("LITESTAR_RELOAD_INCLUDES", ["*.rst2", "*.yml2"]),
            ["--reload-include=*.rst", "--reload-include=*.yml"],
        ),
        (
            ("--reload-exclude", ["*.rst", "*.yml"]),
            ("LITESTAR_RELOAD_EXCLUDES", ["*.rst2", "*.yml2"]),
            ["--reload-exclude=*.rst", "--reload-exclude=*.yml"],
        ),
        (
            ("--wc", 2),
            ("LITESTAR_WEB_CONCURRENCY", 4),
            "--workers=2",
        ),
        (
            ("--fd", 0),
            ("LITESTAR_FILE_DESCRIPTOR", 1),
            "--fd=0",
        ),
        (
            ("--uds", "/run/uvicorn/litestar_test.sock"),
            ("LITESTAR_UNIX_DOMAIN_SOCKET", "/run/uvicorn/litestar_test2.sock"),
            "--uds=/run/uvicorn/litestar_test.sock",
        ),
        (
            ("-d", True),
            ("LITESTAR_DEBUG", False),
            ("LITESTAR_DEBUG", "1"),
        ),
        (
            ("--pdb", True),
            ("LITESTAR_PDB", False),
            ("LITESTAR_PDB", "1"),
        ),
    ),
)
def test_run_command_arguments_precedence(
    cli: Tuple[str, Union[Literal[True], List[str], str]],
    env: Tuple[str, Union[Literal[True], List[str], str]],
    expected: str,
    runner: CliRunner,
    monkeypatch: MonkeyPatch,
    mock_subprocess_run: MagicMock,
    tmp_project_dir: Path,
    create_app_file: CreateAppFileFixture,
    mock_uvicorn_run: MagicMock,
) -> None:
    args = []
    args.extend(["--app", f"{Path('my_app.py').stem}:app"])
    args.extend(["--app-dir", str(Path(tmp_project_dir / "custom_subfolder"))])
    args.extend(["run"])
    create_app_file("my_app.py", directory="custom_subfolder")

    env_name, env_value = env
    cli_name, cli_value = cli

    if env_name:
        if isinstance(env_value, list):
            monkeypatch.setenv(env_name, "".join(env_value))
        else:
            monkeypatch.setenv(env_name, env_value)  # type: ignore[arg-type] # pyright: ignore (reportGeneralTypeIssues)

    if cli_name:
        if cli_value is True:
            args.append(cli_name)
        elif isinstance(cli_value, list):
            for value in cli_value:
                args.extend([cli_name, value])
        else:
            args.extend([cli_name, cli_value])

    result = runner.invoke(cli_command, args)

    assert result.exception is None
    assert result.exit_code == 0

    if cli_name in ["--fd", "--uds"]:
        mock_subprocess_run.assert_not_called()
        if isinstance(expected, list):  # type: ignore[unreachable]
            assert all(_ in mock_uvicorn_run.call_args_list[0].args[0] for _ in expected)  # type: ignore[unreachable]
        else:
            assert mock_uvicorn_run.call_args_list[0].kwargs.get(cli_name.strip("--")) == cli_value

    elif cli_name in ["-d", "--pdb"]:
        assert os.environ.get(expected[0]) == expected[1]

    else:
        mock_subprocess_run.assert_called_once()

        if isinstance(expected, list):  # type: ignore[unreachable]
            assert all(_ in mock_subprocess_run.call_args_list[0].args[0] for _ in expected)  # type: ignore[unreachable]
        else:
            assert expected in mock_subprocess_run.call_args_list[0].args[0]


@pytest.fixture()
def unset_env() -> Generator[None, None, None]:
    initial_env = {**os.environ}
    yield
    for key in os.environ.keys() - initial_env.keys():
        del os.environ[key]

    os.environ.update(initial_env)


@pytest.mark.usefixtures("mock_uvicorn_run", "unset_env")
def test_run_command_debug(
    app_file: Path, runner: CliRunner, monkeypatch: MonkeyPatch, create_app_file: CreateAppFileFixture
) -> None:
    monkeypatch.delenv("LITESTAR_DEBUG", raising=False)
    path = create_app_file("_create_app_with_path.py", content=CREATE_APP_FILE_CONTENT)
    app_path = f"{path.stem}:create_app"
    result = runner.invoke(cli_command, ["--app", app_path, "run", "--debug"])

    assert result.exit_code == 0
    assert os.getenv("LITESTAR_DEBUG") == "1"


@pytest.mark.usefixtures("mock_uvicorn_run", "unset_env")
def test_run_command_quiet_console(
    app_file: Path,
    mocker: MockerFixture,
    runner: CliRunner,
    monkeypatch: MonkeyPatch,
    create_app_file: CreateAppFileFixture,
) -> None:
    mocker.patch.object(core, "isatty", return_value=True)
    mocker.patch.object(_utils, "isatty", return_value=True)
    console = Console(file=io.StringIO(), force_interactive=True)
    monkeypatch.setattr(_utils, "console", console)

    path = create_app_file("_create_app_with_path.py", content=CREATE_APP_FILE_CONTENT)
    app_path = f"{path.stem}:create_app"
    monkeypatch.delenv("LITESTAR_QUIET_CONSOLE", raising=False)
    result = runner.invoke(cli_command, ["--app", app_path, "run"])
    assert result.exit_code == 0
    normal_output = console.file.getvalue()  # type: ignore[attr-defined]
    assert "Using Litestar app from env:" in normal_output
    assert "Starting server process" in result.stdout
    del result
    console = Console(file=io.StringIO(), force_interactive=True)
    monkeypatch.setattr(_utils, "console", console)
    monkeypatch.setenv("LITESTAR_QUIET_CONSOLE", "1")
    assert os.getenv("LITESTAR_QUIET_CONSOLE") == "1"
    result = runner.invoke(cli_command, ["--app", app_path, "run"])
    assert result.exit_code == 0
    quiet_output = console.file.getvalue()  # type: ignore[attr-defined]
    assert "Starting server process" not in result.stdout
    assert "Using Litestar app from env:" not in quiet_output
    console.clear()


@pytest.mark.usefixtures("mock_uvicorn_run", "unset_env")
def test_run_command_custom_app_name(
    app_file: Path,
    runner: CliRunner,
    monkeypatch: MonkeyPatch,
    create_app_file: CreateAppFileFixture,
    mocker: MockerFixture,
) -> None:
    mocker.patch.object(core, "isatty", return_value=True)
    mocker.patch.object(_utils, "isatty", return_value=True)

    console = Console(file=io.StringIO(), force_interactive=True)
    monkeypatch.setattr(_utils, "console", console)

    path = create_app_file("_create_app_with_path.py", content=CREATE_APP_FILE_CONTENT)
    app_path = f"{path.stem}:create_app"
    monkeypatch.delenv("LITESTAR_APP_NAME", raising=False)
    result = runner.invoke(cli_command, ["--app", app_path, "run"])
    assert result.exit_code == 0
    _output = console.file.getvalue()  # type: ignore[attr-defined]
    assert "Using Litestar app from env:" in _output
    console = Console(file=io.StringIO(), force_interactive=True)
    monkeypatch.setattr(_utils, "console", console)
    monkeypatch.setenv("LITESTAR_APP_NAME", "My Stuff")
    assert os.getenv("LITESTAR_APP_NAME") == "My Stuff"
    result = runner.invoke(cli_command, ["--app", app_path, "run"])
    assert result.exit_code == 0
    _output = console.file.getvalue()  # type: ignore[attr-defined]
    assert "Using My Stuff app from env:" in _output


@pytest.mark.usefixtures("mock_uvicorn_run", "unset_env")
def test_run_command_pdb(
    app_file: Path,
    runner: CliRunner,
    monkeypatch: MonkeyPatch,
    create_app_file: CreateAppFileFixture,
) -> None:
    monkeypatch.delenv("LITESTAR_PDB", raising=False)
    path = create_app_file("_create_app_with_path.py", content=CREATE_APP_FILE_CONTENT)
    app_path = f"{path.stem}:create_app"

    with pytest.warns(LitestarWarning):
        result = runner.invoke(cli_command, ["--app", app_path, "run", "--pdb"])

    assert result.exit_code == 0
    assert os.getenv("LITESTAR_PDB") == "1"


@pytest.mark.usefixtures("mock_uvicorn_run", "unset_env")
def test_run_command_without_uvicorn_installed(
    app_file: Path,
    runner: CliRunner,
    monkeypatch: MonkeyPatch,
    create_app_file: CreateAppFileFixture,
    mocker: MockerFixture,
) -> None:
    mocker.patch("litestar.cli.commands.core.UVICORN_INSTALLED", False)
    console_print_mock = mocker.patch("litestar.cli.commands.core.console.print")
    path = create_app_file("_create_app_with_path.py", content=CREATE_APP_FILE_CONTENT)
    app_path = f"{path.stem}:create_app"

    result = runner.invoke(cli_command, ["--app", app_path, "run"])
    assert result.exit_code == 1
    assert any("uvicorn is not installed" in arg for args, kwargs in console_print_mock.call_args_list for arg in args)


@pytest.mark.parametrize("short", [True, False])
def test_version_command(short: bool, runner: CliRunner) -> None:
    result = runner.invoke(cli_command, "version --short" if short else "version")

    assert result.output.strip() == litestar_version.formatted(short=short)


@pytest.mark.usefixtures("mock_uvicorn_run", "unset_env")
def test_run_command_with_server_lifespan_plugin(
    runner: CliRunner, mock_uvicorn_run: MagicMock, create_app_file: CreateAppFileFixture
) -> None:
    path = create_app_file("_create_app_with_path.py", content=APP_FACTORY_FILE_CONTENT_SERVER_LIFESPAN_PLUGIN)
    app_path = f"{path.stem}:create_app"
    result = runner.invoke(cli_command, ["--app", app_path, "run"])

    assert result.exception is None
    assert result.exit_code == 0
    assert "i_run_before_startup_plugin" in result.stdout
    assert "i_run_after_shutdown_plugin" in result.stdout
    assert result.stdout.find("i_run_before_startup_plugin") < result.stdout.find("i_run_after_shutdown_plugin")

    mock_uvicorn_run.assert_called_once_with(
        app=str(app_path),
        host="127.0.0.1",
        port=8000,
        fd=None,
        uds=None,
        factory=True,
        ssl_certfile=None,
        ssl_keyfile=None,
    )


@pytest.mark.parametrize(
    "app_content, schema_enabled, exclude_pattern_list, expected_result_routes_count",
    [
        pytest.param(APP_FILE_CONTENT_ROUTES_EXAMPLE, False, (), 3, id="schema-disabled_no-exclude"),
        pytest.param(
            APP_FILE_CONTENT_ROUTES_EXAMPLE,
            False,
            ("/foo", "/destroy/.*", "/java", "/haskell"),
            2,
            id="schema-disabled_exclude",
        ),
        pytest.param(APP_FILE_CONTENT_ROUTES_EXAMPLE, True, (), 13, id="schema-enabled_no-exclude"),
        pytest.param(
            APP_FILE_CONTENT_ROUTES_EXAMPLE,
            True,
            ("/foo", "/destroy/.*", "/java", "/haskell"),
            12,
            id="schema-enabled_exclude",
        ),
    ],
)
@pytest.mark.xdist_group("cli_autodiscovery")
def test_routes_command_options(
    runner: CliRunner,
    app_content: str,
    schema_enabled: bool,
    exclude_pattern_list: Tuple[str, ...],
    create_app_file: CreateAppFileFixture,
    expected_result_routes_count: int,
) -> None:
    create_app_file("app.py", content=app_content)

    command = "routes"
    if schema_enabled:
        command += " --schema "
    if exclude_pattern_list:
        for pattern in exclude_pattern_list:
            command += f" --exclude={pattern}"

    result = runner.invoke(cli_command, command)
    assert result.exception is None
    assert result.exit_code == 0

    result_routes = [line for line in result.output.splitlines() if "(HTTP)" in line]
    for route in result_routes:
        root_dir = route.split(" ")[0]
        if not schema_enabled:
            assert root_dir != "/api-docs"

        assert root_dir not in exclude_pattern_list

    assert expected_result_routes_count == len(result_routes)


def test_remove_default_schema_routes() -> None:
    routes = [
        "/",
        "/schema",
        "/schema/elements",
        "/schema/oauth2-redirect.html",
        "/schema/openapi.json",
        "/schema/openapi.yaml",
        "/schema/openapi.yml",
        "/schema/rapidoc",
        "/schema/redoc",
        "/schema/swagger",
        "/destroy/all/foo/bar/schema",
        "/foo",
    ]
    http_routes = []
    for route in routes:
        http_route = MagicMock()
        http_route.path = route
        http_routes.append(http_route)

    api_config = MagicMock()
    api_config.openapi_controller.path = "/schema"

    results = _utils.remove_default_schema_routes(http_routes, api_config)  # type: ignore[arg-type]
    assert len(results) == 3
    for result in results:
        words = re.split(r"(^\/[a-z]+)", result.path)
        assert "/schema" not in words


def test_remove_routes_with_patterns() -> None:
    routes = ["/", "/destroy/all/foo/bar/schema", "/foo"]
    http_routes = []
    for route in routes:
        http_route = MagicMock()
        http_route.path = route
        http_routes.append(http_route)

    patterns = ("/destroy", "/pizza", "[]")
    results = _utils.remove_routes_with_patterns(http_routes, patterns)  # type: ignore[arg-type]
    paths = [route.path for route in results]
    assert len(paths) == 2
    for route in ["/", "/foo"]:
        assert route in paths



from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.openapi.spec import OpenAPIType, Schema
from litestar.pagination import ClassicPagination, CursorPagination, OffsetPagination
from litestar.plugins import OpenAPISchemaPlugin

if TYPE_CHECKING:
    from litestar._openapi.schema_generation import SchemaCreator
    from litestar.typing import FieldDefinition


class PaginationSchemaPlugin(OpenAPISchemaPlugin):
    def is_plugin_supported_field(self, field_definition: FieldDefinition) -> bool:
        return field_definition.origin in (ClassicPagination, CursorPagination, OffsetPagination)

    def to_openapi_schema(self, field_definition: FieldDefinition, schema_creator: SchemaCreator) -> Schema:
        if field_definition.origin is ClassicPagination:
            return Schema(
                type=OpenAPIType.OBJECT,
                properties={
                    "items": Schema(
                        type=OpenAPIType.ARRAY,
                        items=schema_creator.for_field_definition(field_definition.inner_types[0]),
                    ),
                    "page_size": Schema(type=OpenAPIType.INTEGER, description="Number of items per page."),
                    "current_page": Schema(type=OpenAPIType.INTEGER, description="Current page number."),
                    "total_pages": Schema(type=OpenAPIType.INTEGER, description="Total number of pages."),
                },
            )

        if field_definition.origin is OffsetPagination:
            return Schema(
                type=OpenAPIType.OBJECT,
                properties={
                    "items": Schema(
                        type=OpenAPIType.ARRAY,
                        items=schema_creator.for_field_definition(field_definition.inner_types[0]),
                    ),
                    "limit": Schema(type=OpenAPIType.INTEGER, description="Maximal number of items to send."),
                    "offset": Schema(type=OpenAPIType.INTEGER, description="Offset from the beginning of the query."),
                    "total": Schema(type=OpenAPIType.INTEGER, description="Total number of items."),
                },
            )

        cursor_schema = schema_creator.not_generating_examples.for_field_definition(field_definition.inner_types[0])
        cursor_schema.description = "Unique ID, designating the last identifier in the given data set. This value can be used to request the 'next' batch of records."

        return Schema(
            type=OpenAPIType.OBJECT,
            properties={
                "items": Schema(
                    type=OpenAPIType.ARRAY,
                    items=schema_creator.for_field_definition(field_definition=field_definition.inner_types[1]),
                ),
                "cursor": cursor_schema,
                "results_per_page": Schema(type=OpenAPIType.INTEGER, description="Maximal number of items to send."),
            },
        )



import dataclasses
from enum import Enum
from typing import Dict, List, Optional
from uuid import UUID

import msgspec
from polyfactory.factories import DataclassFactory
from typing_extensions import NotRequired, Required, TypedDict


class Species(str, Enum):
    DOG = "Dog"
    CAT = "Cat"
    MONKEY = "Monkey"
    PIG = "Pig"


@dataclasses.dataclass
class DataclassPet:
    name: str
    age: float
    species: Species = Species.MONKEY


@dataclasses.dataclass
class DataclassPerson:
    first_name: str
    last_name: str
    id: str
    optional: Optional[str]
    complex: Dict[str, List[Dict[str, str]]]
    pets: Optional[List[DataclassPet]] = None


class TypedDictPerson(TypedDict):
    first_name: Required[str]
    last_name: Required[str]
    id: Required[str]
    optional: NotRequired[Optional[str]]
    complex: Required[Dict[str, List[Dict[str, str]]]]
    pets: NotRequired[Optional[List[DataclassPet]]]


class MsgSpecStructPerson(msgspec.Struct):
    first_name: str
    last_name: str
    id: str
    optional: Optional[str]
    complex: Dict[str, List[Dict[str, str]]]
    pets: Optional[List[DataclassPet]]


@dataclasses.dataclass
class User:
    name: str
    id: UUID


class UserFactory(DataclassFactory[User]):
    __model__ = User


class DataclassPersonFactory(DataclassFactory[DataclassPerson]):
    __model__ = DataclassPerson


class DataclassPetFactory(DataclassFactory[DataclassPet]):
    __model__ = DataclassPet
